<?php
/**
 * @copyright (C) 2021 - 2022 Holger Brandt IT Solutions
 * @license GPL2
*/

if(!defined('ABSPATH')){
    exit;
}

class WADA_Settings
{
    const DELETE_DB_DATA_ON_UNINSTALL = 1;
    const DB_VERSION = 2;
    const LOGGING_LEVEL = 3;
    const LOGGING_LOG_FILE_WARNING_SIZE_MB = 4;
    const LOGGING_FORCED = 5;
    const EVENT_LOG_RETENTION_PERIOD_NUM = 6;
    const EVENT_LOG_RETENTION_PERIOD_UNIT = 7;
    const ANONYMIZE_IP_ADDRESS = 8;

    public static function getMetadataForAllSettings(){
        $metaDataArray = array(
            self::DELETE_DB_DATA_ON_UNINSTALL => (object) array(
                'defaultVal' => '0',
                'getMethod' => 'isDeleteDatabaseDataOnUninstall',
                'setMethod' => 'setIsDeleteDatabaseDataOnUninstall',
                'externalType' => 'bool',
                'storageType' => 'int',
                'field' => 'checkbox',
                'label' => __('Delete data when uninstalling the plugin', 'wp-admin-audit'),
                'title' => __('Checking this option will remove all WP Admin Audit related data, including your event log, configuration settings etc.', 'wp-admin-audit')
            ),
            self::DB_VERSION => (object) array(
                'defaultVal' => '1.1.2.0',
                'getMethod' => 'getDatabaseVersion',
                'setMethod' => 'setDatabaseVersion',
                'externalType' => 'string',
                'storageType' => 'string',
                'field' => 'hidden',
                'label' => __('Database version', 'wp-admin-audit'),
                'title' => __('WP Admin Audit database schema version', 'wp-admin-audit')
            ),
            self::LOGGING_LEVEL => (object) array(
                'defaultVal' =>  WADA_Constants::LOG_LEVEL_INFO,
                'getMethod' => 'getLoggingLevel',
                'setMethod' => 'setLoggingLevel',
                'externalType' => 'int',
                'storageType' => 'int',
                'field' => 'select',
                'label' => __('Logging level', 'wp-admin-audit'),
                'title' => __("The logging level determines which kind of internal plugin events (e.g. an error while storing some data) should be written to WP Admin Audit's log file. Note this is independent from your site's event log tracking your admin's activities. The purpose of the log file is that you can provide it to the WP Admin Audit tech support for review in case the plugin is not working as expected.", 'wp-admin-audit'),
                'selectOptions' => array(
                    WADA_Constants::LOG_LEVEL_OFF =>  __("Disabled", 'wp-admin-audit'),
                    WADA_Constants::LOG_LEVEL_ERROR =>  __("Errors only", 'wp-admin-audit'),
                    WADA_Constants::LOG_LEVEL_INFO =>  __("Normal", 'wp-admin-audit'),
                    WADA_Constants::LOG_LEVEL_DEBUG =>  __("Max. Logging (Debug)", 'wp-admin-audit')
                ),
                'disabledSelectOptions' => array(),
                'multiSelect' => false
            ),
            self::LOGGING_LOG_FILE_WARNING_SIZE_MB => (object) array(
                'defaultVal' =>  '50',
                'getMethod' => 'getLogFileSizeWarningLevel',
                'setMethod' => 'setLogFileSizeWarningLevel',
                'externalType' => 'int',
                'storageType' => 'int',
                'field' => 'input',
                'label' => __('Log file warning size limit (MB)', 'wp-admin-audit'),
                'title' => __('When this size of the log file (in megabyte) is exceeded a warning message is shown', 'wp-admin-audit')
            ),
            self::LOGGING_FORCED => (object) array(
                'defaultVal' =>  '0',
                'getMethod' => 'isLoggingForced',
                'setMethod' => 'setIsLoggingForced',
                'externalType' => 'bool',
                'storageType' => 'int',
                'field' => 'checkbox',
                'label' => __('Force logging', 'wp-admin-audit'),
                'title' => __('Deactivates checks whether log directory is existing and log file is writable. Use with caution! This can not work e.g. if your log path is configured wrong.', 'wp-admin-audit')
            ),
            self::EVENT_LOG_RETENTION_PERIOD_NUM => (object) array(
                'defaultVal' =>  '30',
                'getMethod' => 'getRetentionPeriodNum',
                'setMethod' => 'setRetentionPeriodNum',
                'externalType' => 'int',
                'storageType' => 'int',
                'field' => 'input',
                'label' => __('Retention period', 'wp-admin-audit'),
                'title' => __('How long events should be kept in the event log before being deleted', 'wp-admin-audit')
            ),
            self::EVENT_LOG_RETENTION_PERIOD_UNIT => (object) array(
                'defaultVal' =>  'd',
                'getMethod' => 'getRetentionPeriodUnit',
                'setMethod' => 'setRetentionPeriodUnit',
                'externalType' => 'string',
                'storageType' => 'string',
                'field' => 'select',
                'selectOptions' => array(
                    'd' =>  __("Days", 'wp-admin-audit'),
                    'm' =>  __("Months", 'wp-admin-audit'),
                    'y' =>  __("Years", 'wp-admin-audit')
                ),
                'label' => __('Retention period', 'wp-admin-audit'),
                'title' => __('How long events should be kept in the event log before being deleted', 'wp-admin-audit')
            ),
            self::ANONYMIZE_IP_ADDRESS => (object) array(
            'defaultVal' =>  '1',
            'getMethod' => 'isAnonymizeIPAddress',
            'setMethod' => 'setIsAnonymizeIPAddress',
            'externalType' => 'bool',
            'storageType' => 'int',
            'field' => 'checkbox',
            'label' => __('Anonymize IP Address', 'wp-admin-audit'),
            'title' => __('In the context of the data stored in the event log: removes part of the IP address (for anonymization of the user) to comply with privacy regulations', 'wp-admin-audit')
        ),
        );
        return (object) $metaDataArray;
    }

    public static function renderSettingField($settingId, $asTableRow=true, $useDefaultsIfNeeded=true, $echoImmediately=true, $renderChildIfApplicable=true, $options=array()){
        $returnVal = null;
        $metaData = self::getMetadataForSetting($settingId);
        if($metaData){
            $name = $settingId;
            $options['title'] = $metaData->title;
            $options['title_as_info_icon'] = true;
            $options['id'] = 'setting'.$name;
            if($asTableRow){
                $options['render_as_table_row'] = true;
            }
            if(!$echoImmediately){
                $options['return_as_str'] = true;
            }
            if($renderChildIfApplicable && property_exists($metaData, 'hasChild')){
                $childReturnVal = self::renderSettingField($metaData->hasChild, false, $useDefaultsIfNeeded ? true : null, false);
                $options['html_suffix'] = $childReturnVal;
            }
            $label = property_exists($metaData, 'label') ? $metaData->label : null;
            $defaultValue = property_exists($metaData, 'defaultVal') ? $metaData->defaultVal : null;
            $value = property_exists($metaData, 'getMethod') && is_callable(array(__CLASS__, $metaData->getMethod)) ? call_user_func(array(__CLASS__, $metaData->getMethod), $defaultValue)  : self::getSetting($settingId, $useDefaultsIfNeeded ? $defaultValue : null);
            WADA_Log::debug('Render '.$name.', label: '.$label.', value: '.$value);
            switch($metaData->field){
                case 'checkbox':
                    $returnVal = WADA_HtmlUtils::checkboxField($name, $label, $value, $options);
                    break;
                case 'input':
                    $returnVal = WADA_HtmlUtils::inputField($name, $label, $value, $options);
                    break;
                case 'password':
                    $returnVal = WADA_HtmlUtils::passwordField($name, $label, $value, $options);
                    break;
                case 'textarea':
                    $returnVal = WADA_HtmlUtils::textareaField($name, $label, $value, null, null, $options);
                    break;
                case 'select':
                    $returnVal = WADA_HtmlUtils::selectField($name, $label, $value, $metaData->selectOptions,
                        property_exists($metaData, 'disabledSelectOptions') ? $metaData->disabledSelectOptions : array(),
                        property_exists($metaData, 'multiSelect') ? $metaData->multiSelect : false,
                        $options);
                    break;
                case 'hidden':
                    $returnVal = WADA_HtmlUtils::hiddenField($name, $value, $options);
                    break;
            }
        }
        return $returnVal;
    }

    public static function getMetadataForSetting($settingId){
        $metaData = self::getMetadataForAllSettings(); // get metadata from all settings
        if(property_exists($metaData, $settingId)) {
            $metaData->$settingId->settingId = $settingId;
           return $metaData->$settingId; // pull out this setting's metadata
        }
        return false;
    }

    public static function doesSettingExist($settingId){
        global $wpdb;
        $query = 'SELECT setting_value FROM '.WADA_Database::tbl_settings().' WHERE id = %d';
        $settingValue = $wpdb->get_var($wpdb->prepare($query, $settingId));
        if(is_null($settingValue)){
            return false;
        }
        return true;
    }

    public static function isDeleteDatabaseDataOnUninstall($default=false){
        return self::getSetting(self::DELETE_DB_DATA_ON_UNINSTALL, $default);
    }

    public static function setIsDeleteDatabaseDataOnUninstall($deleteDbDataOnUninstall){
        return self::setSetting(self::DELETE_DB_DATA_ON_UNINSTALL, $deleteDbDataOnUninstall);
    }

    public static function getDatabaseVersion($default=null, $storeInDbIfNotExists=false){
        return self::getSetting(self::DB_VERSION, $default ?: '1.0', $storeInDbIfNotExists);
    }

    public static function setDatabaseVersion($dbVersion){
        return self::setSetting(self::DB_VERSION, $dbVersion);
    }

    public static function getLoggingLevel($default=WADA_Constants::LOG_LEVEL_INFO) {
        return self::getSetting(self::LOGGING_LEVEL, $default);
    }

    public static function setLoggingLevel($loggingLevel) {
        return self::setSetting(self::LOGGING_LEVEL, $loggingLevel);
    }

    public static function getLogFileSizeWarningLevel($default=50) {
        return self::getSetting(self::LOGGING_LOG_FILE_WARNING_SIZE_MB, $default);
    }

    public static function setLogFileSizeWarningLevel($warningLevelInMb) {
        return self::setSetting(self::LOGGING_LOG_FILE_WARNING_SIZE_MB, $warningLevelInMb);
    }

    public static function isLoggingForced($default=0) {
        return self::getSetting(self::LOGGING_FORCED, $default);
    }

    public static function setIsLoggingForced($isLoggingForced) {
        return self::setSetting(self::LOGGING_FORCED, $isLoggingForced);
    }

    public static function getRetentionPeriodInDays($retentionPeriodNumber = null, $retentionPeriodUnit = null){
        $retentionPeriodNumber = (is_null($retentionPeriodNumber) ? absint(self::getRetentionPeriodNum()) : $retentionPeriodNumber);
        $retentionPeriodUnit = (is_null($retentionPeriodUnit) ? absint(self::getRetentionPeriodUnit()) : $retentionPeriodUnit);
        $res = $retentionPeriodNumber;
        switch($retentionPeriodUnit){
            case 'd':
                $res = $retentionPeriodNumber;
                break;
            case 'm':
                $res = $retentionPeriodNumber*31;
                break;
            case 'y':
                $res = $retentionPeriodNumber*365;
                break;
            default:
                $res = $retentionPeriodNumber;
        }
        return $res;
    }

    public static function setRetentionPeriod($retentionPeriodNumber, $retentionPeriodUnit = 'd') {
        $retentionPeriodNumber = absint($retentionPeriodNumber);
        $retDays = self::getRetentionPeriodInDays($retentionPeriodNumber, $retentionPeriodUnit);
        if(absint(WADA_Version::getFtSetting(WADA_Version::FT_ID_RET)) < $retDays || (absint(WADA_Version::getFtSetting(WADA_Version::FT_ID_RET))<100000 && $retDays == 0)){
            $retentionPeriodNumber = absint(WADA_Version::getFtSetting(WADA_Version::FT_ID_RET)) ;
            $retentionPeriodUnit = 'd';
        }
        self::setRetentionPeriodUnit($retentionPeriodUnit);
        return self::setSetting(self::EVENT_LOG_RETENTION_PERIOD_NUM, $retentionPeriodNumber);
    }

    public static function getRetentionPeriodNum($default=30){
        return absint(self::getSetting(self::EVENT_LOG_RETENTION_PERIOD_NUM, $default));
    }

    public static function setRetentionPeriodNum($retentionPeriodNumber) {
        $retentionPeriodNumber = absint($retentionPeriodNumber);
        return self::setSetting(self::EVENT_LOG_RETENTION_PERIOD_NUM, $retentionPeriodNumber);
    }

    public static function getRetentionPeriodUnit($default='d'){
        return self::getSetting(self::EVENT_LOG_RETENTION_PERIOD_UNIT, $default);
    }

    public static function setRetentionPeriodUnit($retentionPeriodUnit) {
        $retentionPeriodUnit = strtolower($retentionPeriodUnit);
        $allowedValues = array('d', 'm', 'y');
        if(!in_array($retentionPeriodUnit, $allowedValues) || (absint(WADA_Version::getFtSetting(WADA_Version::FT_ID_RET)) < 100000)){
            $retentionPeriodUnit = 'd';
        }
        return self::setSetting(self::EVENT_LOG_RETENTION_PERIOD_UNIT, $retentionPeriodUnit);
    }

    public static function isAnonymizeIPAddress($default=1) {
        return self::getSetting(self::ANONYMIZE_IP_ADDRESS, $default);
    }

    public static function setIsAnonymizeIPAddress($anonymizeIPAddress) {
        return self::setSetting(self::ANONYMIZE_IP_ADDRESS, $anonymizeIPAddress);
    }

    public static function getAllSettings($useNameAsIndex = false){
        $class = new ReflectionClass(__CLASS__);
        $constants = array_flip($class->getConstants());
        $settings = array();
        $metaData = self::getMetadataForAllSettings();
        foreach ($constants as $settingId => $settingName) {
            if(property_exists($metaData, $settingId)) {
                $getValueMethod = $metaData->$settingId->getMethod;
                $value = is_callable(array(__CLASS__, $getValueMethod)) ? call_user_func(array(__CLASS__, $getValueMethod)) : null;
                $settingObj = (object)array('id' => $settingId, 'name' => $settingName, 'value' => $value, 'metaData' => $metaData->$settingId);
                if($useNameAsIndex){
                    $settings[$settingName] = $settingObj;
                }else{
                    $settings[$settingId] = $settingObj;
                }
            }
        }
        //WADA_Log::debug('getAllSettings settings: '.print_r($settings, true));
        return $settings;
    }

    protected static function getSetting($settingId, $default=null, $storeInDbIfNotExists=false){
        global $wpdb;
        $query = 'SELECT setting_value FROM '.WADA_Database::tbl_settings().' WHERE id = %d';
        $settingValue = $wpdb->get_var($wpdb->prepare($query, $settingId));
        if(is_null($settingValue)){
            if(!is_null($default) && $storeInDbIfNotExists){
                self::setSetting($settingId, $default);
            }
            return $default;
        }
        $metaData = self::getMetadataForSetting($settingId);
        if($metaData){
            if($metaData->externalType == 'bool' && $metaData->storageType == 'int'){
                $settingValue = (intval($settingValue) > 0);
            }elseif($metaData->externalType == 'int'){
                $settingValue = intval($settingValue);
            }
        }
        return $settingValue;
    }

    protected static function setSetting($settingId, $value){
        $metaData = self::getMetadataForSetting($settingId);
        if($metaData){
            if($metaData->externalType == 'bool' && $metaData->storageType == 'int'){
                if(is_bool($value)===true){
                    $value = $value ? '1' : '0';
                }else{
                    $value = intval($value);
                }
            }elseif($metaData->storageType == 'int'){
                $value = intval($value);
            }
        }

        $value = strval($value); // convert (back) to string since that is what the DB has
        global $wpdb;
        return $wpdb->replace(WADA_Database::tbl_settings(),
            array(
                'id' => intval($settingId),
                'setting_value' => $value
            ),
            array(
               '%d',
               '%s'
            )
        );
    }
}