<?php
/**
 * @copyright (C) 2021 - 2022 Holger Brandt IT Solutions
 * @license GPL2
*/

if(!defined('ABSPATH')){
    exit;
}

class WADA_Sensor_Post extends WADA_Sensor_Base
{
    protected $priorPost = null;
    public function __construct($sensorGroup = WADA_Sensor_Base::GRP_POST){
        parent::__construct($sensorGroup);
    }

    public function registerSensor(){
        add_action('pre_post_update', array($this, 'onPostAboutToUpdate'), 10, 2);
        add_action('wp_insert_post', array($this, 'onPostInsert'), 10, 3);
        add_action('transition_post_status', array($this, 'onPostTransitionStatus'), 10, 3);
        add_action('delete_post', array($this, 'onPostDelete'), 10, 2);
        //WADA_Log::debug('Active sensors for group '.$this->sensorGroup.': '.print_r($this->activeSensors, true));
    }

    protected function getWP_PostAttributesToRecord(){
        return array(
            'ID', 'post_author', 'post_date', 'post_date_gmt', 'post_content', 'post_title', 'post_excerpt',
            'post_status', 'comment_status', 'ping_status', 'post_password', 'post_name',
            'to_ping', 'pinged',
            'post_modified', 'post_modified_gmt', 'post_content_filtered', 'post_parent', 'guid', 'menu_order',
            'post_type', 'post_mime_type', 'comment_count', 'filter');
    }

    /**
     * @param WP_Post $post
     */
    protected function isPostEventToBeIgnored($post){
        $supportedPostEvents = array('page', 'post'); // not putting attachment here -> considered media event
        if(in_array($post->post_type, $supportedPostEvents)){
            return false;
        }
        WADA_Log::debug('Ignore post event of type '.$post->post_type.' (ID '.$post->ID.')');
        return true;
    }

    /**
     * @param int $postId
     * @param array $data
     */
    public function onPostAboutToUpdate($postId, $data){
        $this->priorPost = get_post(intval($postId));
    }

    /**
     * @param string $newStatus
     * @param string $oldStatus
     * @param WP_Post $post
     * @return bool|int
     */
    public function onPostTransitionStatus($newStatus, $oldStatus, $post){
        if($this->isPostEventToBeIgnored($post)) return $this->skipEvent('[POST TRANSITION PSEUDO]', false);
        if($newStatus === $oldStatus) return $this->skipEvent('[POST TRANSITION PSEUDO]', false, 'No status change (both '.$newStatus.')');
        WADA_Log::debug('onPostTransitionStatus status (old->new): '.$oldStatus.' -> '.$newStatus. ' for post ID '.$post->ID.', post: '.print_r($post, true));

        $sensorId = 0;
        if(($newStatus !== 'auto-draft' && $newStatus !== 'trash') && ($oldStatus === 'new' || $oldStatus === 'auto-draft')){
            $sensorId = WADA_Sensor_Base::EVT_POST_CREATE;
        }else if(($newStatus === 'publish') && ($oldStatus === 'pending' || $oldStatus === 'draft' || $oldStatus === 'future' || $oldStatus === 'trash')){
            $sensorId = WADA_Sensor_Base::EVT_POST_PUBLISHED;
        }else if($oldStatus === 'publish'){
            $sensorId = WADA_Sensor_Base::EVT_POST_UNPUBLISHED;
        }else if($newStatus === 'trash'){
            $sensorId = WADA_Sensor_Base::EVT_POST_TRASHED;
        }

        if(!$sensorId) return $this->skipEvent('[POST TRANSITION PSEUDO]', false, 'Seems we are not interested in (old->new): '.$oldStatus.' -> '.$newStatus.' for post ID '.$post->ID);
        if(!$this->isActiveSensor($sensorId)) return $this->skipEvent($sensorId);

        WADA_Log::debug('onPostTransitionStatus Looking back: '.print_r($this->eventTracker, true));
        $postBeforeStatusChange = get_post($post->ID);
        // $infos = WADA_CompUtils::getChangedAttributes($postBeforeStatusChange, $post, self::getWP_PostAttributesToRecord());
        $infos = array_merge(array(self::getEventInfoElement('POST_STATUS_TRANSITION', $newStatus, $oldStatus)), $this->getPostEventDetails($post, $this->priorPost)); // record the full thing

        $eventData = array('infos' => $infos);
        return $this->storePostEvent($sensorId, $eventData, $post->ID);
    }

    /**
     * @param int $postId
     * @param WP_Post $post
     * @param bool $update
     */
    public function onPostInsert($postId, $post, $update){
        WADA_Log::debug('onPostInsert');
        if(!$this->isActiveSensor(self::EVT_POST_CREATE)
            && !$this->isActiveSensor(self::EVT_POST_UPDATE)) return $this->skipEvent(self::EVT_POST_UPDATE);

        if($this->isPostEventToBeIgnored($post)) return $this->skipEvent(self::EVT_POST_UPDATE, false);

        WADA_Log::debug('onPostInsert Looking back: '.print_r($this->eventTracker, true));
        WADA_Log::debug('onPostInsert for postId '.$postId.' (update: '.($update?'yes':'no').'), post: '.print_r($post, true));
        if($post->post_status === 'auto-draft'){
            return $this->skipEvent(self::EVT_POST_UPDATE, false, 'Skip recording auto-draft as update for post ID '.$post->ID);
        }

        $pastPostEvent = array_search($post->ID, array_column($this->eventTracker, 'post_id'));
        if($pastPostEvent !== false){
            $pastPostEventSensorId = $this->eventTracker[$pastPostEvent]['sensor_id'];
            WADA_Log::debug('onPostInsert Found past event in tracker, sensor ID: '.$pastPostEventSensorId);
            if($pastPostEventSensorId === WADA_Sensor_Base::EVT_POST_CREATE
                || $pastPostEventSensorId === WADA_Sensor_Base::EVT_POST_PUBLISHED
                || $pastPostEventSensorId === WADA_Sensor_Base::EVT_POST_UNPUBLISHED
                || $pastPostEventSensorId === WADA_Sensor_Base::EVT_POST_TRASHED){
                return $this->skipEvent(self::EVT_POST_UPDATE, false, 'Skip recording update since we recorded status transition for post ID '.$post->ID);
            }
        }

        $infos = $this->getPostEventDetails($post, $this->priorPost);
        $eventData = array('infos' => $infos);
        return $this->storePostEvent(self::EVT_POST_UPDATE, $eventData, $postId);
    }

    /**
     * @param int $postId
     * @param WP_Post $post
     */
    public function onPostDelete($postId, $post){
        WADA_Log::debug('onPostDelete');
        if(!$this->isActiveSensor(self::EVT_POST_DELETE)) return $this->skipEvent(self::EVT_POST_DELETE);

        if($this->isPostEventToBeIgnored($post)) return $this->skipEvent(self::EVT_POST_DELETE, false);

        WADA_Log::debug('onPostDelete Looking back: '.print_r($this->eventTracker, true));
        WADA_Log::debug('onPostDelete for postId '.$postId.', post: '.print_r($post, true));

        $infos = $this->getPostEventDetails($post, $this->priorPost);
        $eventData = array('infos' => $infos);
        return $this->storePostEvent(self::EVT_POST_DELETE, $eventData, $postId);
    }

    /**
     * @param WP_Post $currentPost
     * @param WP_Post|null $priorPost
     * @return array
     */
    protected function getPostEventDetails($currentPost, $priorPost=null){
        $infos = array();
        $attributesToRecord = $this->getWP_PostAttributesToRecord();
        if($currentPost){
            foreach($attributesToRecord as $attribute){
                if(isset($currentPost->$attribute) && $priorPost && isset($priorPost->$attribute)) {
                    $infos[] = self::getEventInfoElement($attribute, $currentPost->$attribute, $priorPost->$attribute);
                }else if(isset($currentPost->$attribute)) {
                    $infos[] = self::getEventInfoElement($attribute, $currentPost->$attribute);
                }
            }
        }
        return $infos;
    }

    /**
     * @param int $userId
     * @param int $targetObjectId
     * @param string|null $targetObjectType
     * @return array
     */
    protected function getEventDefaults($userId = 0, $targetObjectId = 0, $targetObjectType = self::OBJ_TYPE_CORE_POST){
        // change to parent function is that we default to passing in the object type of WP Post
        return parent::getEventDefaults($userId, $targetObjectId, $targetObjectType);
    }

    /**
     * @param int $sensorId
     * @param array $eventData
     * @param int $targetPostId
     * @return bool
     */
    protected function storePostEvent($sensorId, $eventData = array(), $targetPostId = 0){
        $executingUserId = get_current_user_id();
        $event = (object)(array_merge($this->getEventDefaults($executingUserId, $targetPostId), $eventData));
        // WADA_Log::debug('storePostEvent (sensor: '.$sensorId.'), event: '.print_r($event, true));
        $res = $this->storeEvent($sensorId, $event);
        if($res){
            $this->eventTracker[] = array('post_id'=>$targetPostId, 'sensor_id'=>$sensorId);
        }
        return $res;
    }

}