<?php
/**
 * @copyright (C) 2021 - 2022 Holger Brandt IT Solutions
 * @license GPL2
*/

if(!defined('ABSPATH')){
    exit;
}

class WADA_Sensor_User extends WADA_Sensor_Base
{
    public function __construct(){
        parent::__construct(WADA_Sensor_Base::GRP_USER);
    }

    public function registerSensor(){
        add_action('user_register', array($this, 'onUserRegistration'), 10, 2);
        add_action('wp_login', array($this, 'onUserLogin'), 10, 2 );
        add_action('wp_login_failed', array($this, 'onUserLoginFailed'), 10, 2);
        add_action('wp_logout', array($this, 'onUserLogout'));
        add_action('profile_update', array($this, 'onUserUpdate'), 10, 3);
        add_action('delete_user', array($this, 'onUserDelete'), 10, 3);
        add_action('password_reset', array($this, 'onUserPasswordReset'), 10, 2);
        //WADA_Log::debug('Active sensors for group '.$this->sensorGroup.': '.print_r($this->activeSensors, true));
    }

    /**
     * @param int $userId
     * @param int|null $reassignUserId
     * @param WP_User $user
     */
    public function onUserDelete($userId, $reassignUserId = null, $user = null){
        if(!$this->isActiveSensor(self::EVT_USER_DELETE)) return $this->skipEvent(self::EVT_USER_DELETE);
        WADA_Log::debug('onUserDelete for userId '.$userId.' reassign user id: '.$reassignUserId.', user: '.print_r($user, true));
        $infos = array();
        if(!$user){
            $user = get_user_by('id', $userId);
        }
        $attributesToRecord = WADA_UserUtils::getTopLevelAttributes();
        $infos[] = self::getEventInfoElement('USER_ID', $userId);
        foreach($attributesToRecord as $attribute){
            if(isset($user->$attribute)) {
                $infos[] = self::getEventInfoElement('DEL_DATA_' . $attribute, $user->$attribute);
            }
        }
        if($reassignUserId && intval($reassignUserId)>0 && $userId != $reassignUserId){
            $infos[] = self::getEventInfoElement('REASSIGN_USER_ID', $reassignUserId);
        }
        $eventData = array('infos' => $infos);
        $executingUserId = get_current_user_id();
        $targetObjectId = $userId;
        return $this->storeUserEvent(self::EVT_USER_DELETE, $eventData, $executingUserId, $targetObjectId);
    }

    /**
     * @param int $userId
     * @param WP_User $oldUserData
     * @param array $userData
     * @return bool
     */
    public function onUserUpdate($userId, $oldUserData, $userData = array()){
        if(!$this->isActiveSensor(self::EVT_USER_UPDATE)) return $this->skipEvent(self::EVT_USER_UPDATE);
        $wpUser = get_user_by('id', $userId);
        WADA_Log::debug('onUserUpdate for userId '.$userId.' old user data: '.print_r($oldUserData, true).', user data: '.print_r($userData, true).', new user data: '.print_r($wpUser, true));
        $changedAttributes = WADA_UserUtils::getChangedAttributes($oldUserData, $wpUser);
        WADA_Log::debug('onUserUpdate changed attributes: '.print_r($changedAttributes, true));
        $eventData = array('infos' => $changedAttributes);
        $executingUserId = get_current_user_id();
        $targetObjectId = $userId;
        return $this->storeUserEvent(self::EVT_USER_UPDATE, $eventData, $executingUserId, $targetObjectId);
    }

    /**
     * @param int $userId
     * @param array $userData
     * @return bool
     */
    public function onUserRegistration($userId, $userData){
        if(!$this->isActiveSensor(self::EVT_USER_REGISTRATION)) return $this->skipEvent(self::EVT_USER_REGISTRATION);
        WADA_Log::debug('onUserRegistration for userId '.$userId.' with user data: '.print_r($userData, true));

        $newUserMetaData = get_userdata($userId);
        WADA_Log::debug('onUserRegistration meta data for userId '.$userId.': '.print_r($newUserMetaData, true));

        if($newUserMetaData && property_exists($newUserMetaData, 'roles')){
            $newUserRoles = is_array($newUserMetaData->roles) ?
                implode(', ', $newUserMetaData->roles)
                : (is_string($newUserMetaData->roles) ? $newUserMetaData->roles : null);
        }else{
            $newUserRoles = null;
        }

        $eventData = array('infos' => array(
            self::getEventInfoElement('user_id', $userId), // this is somewhat redundant to target object id, but we are okay with it
            self::getEventInfoElement('user_login', (array_key_exists('user_login', $userData) ? $userData['user_login'] : null)),
            self::getEventInfoElement('user_email', (array_key_exists('user_email', $userData) ? $userData['user_email'] : null)),
            self::getEventInfoElement('roles', $newUserRoles)
        ));

        $targetObjectId = $userId;
        $loggedInUserId = get_current_user_id();
        if($loggedInUserId > 0 && $loggedInUserId != $userId){
            WADA_Log::debug('onUserRegistration User '.$userId.' was created by user '.$loggedInUserId);
            // if the logged-in user is not the user that was created, then we need to care about it
            // that is the admin that created the user via the backend
            $executingUserId = $loggedInUserId;
        }else{
            $executingUserId = $userId;
        }

        return $this->storeUserEvent(self::EVT_USER_REGISTRATION, $eventData, $executingUserId, $targetObjectId);
    }

    /**
     * @param string $userLogin
     * @param WP_User $user
     * @return bool
     */
    public function onUserLogin($userLogin, $user){
        if(!$this->isActiveSensor(self::EVT_USER_LOGIN)) return $this->skipEvent(self::EVT_USER_LOGIN);
        WADA_Log::debug('onUserLogin for userLogin '.$userLogin.' with user: '.print_r($user, true));
        return $this->storeUserEvent(self::EVT_USER_LOGIN, array(), $user->ID);
    }

    /**
     * @param string $userLogin
     * @param WP_Error $error
     * @return bool
     */
    public function onUserLoginFailed($userLogin, $error){
        if(!$this->isActiveSensor(self::EVT_USER_LOGIN_FAILED)) return $this->skipEvent(self::EVT_USER_LOGIN_FAILED);
        WADA_Log::debug('onUserLoginFailed for userLogin '.$userLogin.' with error: '.print_r($error, true));
        $errorCode = $error->get_error_code();
        $targetUserId = 0;
        $errorMessage = $errorCode . '|' . $error->get_error_message();
        if($errorCode === 'incorrect_password'){
            $errorMessage = __('Incorrect password', 'wp-admin-audit');
            $user = get_user_by('login', $userLogin);
            if($user){
                $targetUserId = $user->ID;
            }
        }else if($errorCode === 'invalid_username'){
            $errorMessage = __('Invalid username', 'wp-admin-audit');
        }
        $eventData = array('infos' => array(
            self::getEventInfoElement('username', $userLogin),
            self::getEventInfoElement('error', $errorMessage)
        ));
        return $this->storeUserEvent(self::EVT_USER_LOGIN_FAILED, $eventData, 0, $targetUserId);
    }

    /**
     * @param int $userId
     * @return bool
     */
    public function onUserLogout($userId){
        if(!$this->isActiveSensor(self::EVT_USER_LOGOUT)) return $this->skipEvent(self::EVT_USER_LOGOUT);
        WADA_Log::debug('onUserLogout for userLogin '.$userId);
        return $this->storeUserEvent(self::EVT_USER_LOGOUT, array(), $userId);
    }

    /**
     * @param WP_User $user
     * @param string $newPassword
     */
    public function onUserPasswordReset($user, $newPassword){
        if(!$this->isActiveSensor(self::EVT_USER_PASSWORD_RESET)) return $this->skipEvent(self::EVT_USER_PASSWORD_RESET);
        WADA_Log::debug('onUserPasswordReset for user '.print_r($user, true));
        $loggedInUserId = get_current_user_id();
        return $this->storeUserEvent(self::EVT_USER_PASSWORD_RESET, array(), $loggedInUserId, $user->ID);
    }

    /**
     * @param int $userId
     * @param int $targetObjectId
     * @param string|null $targetObjectType
     * @return array
     */
    protected function getEventDefaults($userId = 0, $targetObjectId = 0, $targetObjectType = self::OBJ_TYPE_CORE_USER){
        // change to parent function is that we default to passing in the object type of WP User
        return parent::getEventDefaults($userId, $targetObjectId, $targetObjectType);
    }

    /**
     * @param int $sensorId
     * @param array $eventData
     * @param int $executingUserId
     * @param int $targetUserId
     * @return bool
     */
    protected function storeUserEvent($sensorId, $eventData = array(), $executingUserId = 0, $targetUserId = 0){
        $event = (object)(array_merge($this->getEventDefaults($executingUserId, $targetUserId), $eventData));
        return $this->storeEvent($sensorId, $event);
    }

}