<?php
/**
 * @copyright (C) 2021 - 2022 Holger Brandt IT Solutions
 * @license GPL2
*/

if(!defined('ABSPATH')){
    exit;
}

require_once 'Constants.php';
require_once 'Application/Log.php';
require_once 'Application/Database.php';
require_once 'Application/Settings.php';
require_once 'Application/Version.php';

class WADA_Setup
{
    public function installOrUpdate(){
        WADA_Log::initFile(); // we need to make sure that on installation the log files get created
        WADA_Log::installLog('installOrUpdate');
        $this->setupInitialDatabaseIfNeeded();
        $this->runUpdatesAndMigrations();
        $this->scheduleEvents();
    }

    protected function setupInitialDatabaseIfNeeded(){
        WADA_Log::installLog('setupInitialDatabaseIfNeeded');
        $this->createTablesIfNotExisting();
        $this->createIndexesIfNotExisting();
        $this->setupStandardSettingsIfNeeded();
        $this->setupSensorsIfNeeded();
    }

    public function createTablesIfNotExisting(){
        WADA_Log::installLog('createTablesIfNotExisting');
        global $wpdb;
        $charsetCollate = $wpdb->get_charset_collate();

        $tblSQLs = array();
        $tblSQLs[] = "CREATE TABLE IF NOT EXISTS ".WADA_Database::tbl_sensors()." (
                      id INT NOT NULL,
                      extension_id INT NOT NULL DEFAULT 0,
                      severity INT NULL,
                      active TINYINT(1) NOT NULL DEFAULT 1,
                      name VARCHAR(45) NOT NULL,
                      description TEXT NOT NULL,
                      event_group VARCHAR(45) NULL,
                      event_category VARCHAR(45) NULL,
                      PRIMARY KEY (id)
                      ) ".$charsetCollate.";";
        $tblSQLs[] = "CREATE TABLE IF NOT EXISTS ".WADA_Database::tbl_events()." (
                      id INT NOT NULL AUTO_INCREMENT,
                      occurred_on TIMESTAMP NOT NULL,
                      sensor_id INT NOT NULL,
                      site_id INT NOT NULL,
                      user_id INT NOT NULL,
                      user_name VARCHAR(255) NULL,
                      user_email VARCHAR(255) NULL,
                      object_type VARCHAR(45) NULL,
                      object_id INT NULL,
                      source_ip VARCHAR(39) NULL,
                      source_client VARCHAR(255) NULL,
                      check_value_head VARCHAR(128) NOT NULL,
                      check_value_full VARCHAR(128) NULL,
                      replication_done TINYINT(1) NOT NULL DEFAULT 0,
                      PRIMARY KEY (id)
                      ) ".$charsetCollate.";";
        $tblSQLs[] = "CREATE TABLE IF NOT EXISTS ".WADA_Database::tbl_event_infos()." (
                      id INT NOT NULL AUTO_INCREMENT,
                      event_id INT NOT NULL,
                      info_key VARCHAR(45) NOT NULL,
                      info_value TEXT NULL,
                      prior_value TEXT NULL,
                      PRIMARY KEY (id)
                      ) ".$charsetCollate.";";
        $tblSQLs[] = "CREATE TABLE IF NOT EXISTS ".WADA_Database::tbl_extensions()." (
                      id INT NOT NULL AUTO_INCREMENT,
                      active TINYINT(1) NOT NULL DEFAULT 0,
                      name VARCHAR(45) NOT NULL,
                      plugin_folder VARCHAR(90) NOT NULL,
                      ext_api_version INT NOT NULL,
                      PRIMARY KEY (id)
                      ) ".$charsetCollate.";";
        $tblSQLs[] = "CREATE TABLE IF NOT EXISTS ".WADA_Database::tbl_settings()." (
                      id INT NOT NULL,
                      setting_value TEXT NULL,
                      PRIMARY KEY (id)
                      ) ".$charsetCollate.";";
        $tblSQLs[] = "CREATE TABLE IF NOT EXISTS ".WADA_Database::tbl_notifications()." (
                      id INT NOT NULL AUTO_INCREMENT,
                      active TINYINT(1) NOT NULL DEFAULT 1,
                      notification_type INT NOT NULL,
                      observed_sensor_id INT NULL,
                      observed_user_id INT NULL,
                      target_type INT NULL,
                      target_user_id INT NULL,
                      target_group_id INT NULL,
                      target_value TEXT NULL,
                      PRIMARY KEY (id)
                      ) ".$charsetCollate.";";
        $tblSQLs[] = "CREATE TABLE IF NOT EXISTS ".WADA_Database::tbl_event_notifications()." (
                      id INT NOT NULL AUTO_INCREMENT,
                      event_id INT NOT NULL,
                      notification_id INT NOT NULL,
                      sent TINYINT(0) NOT NULL DEFAULT 0,
                      sent_on TIMESTAMP NULL,
                      success TINYINT(0) NOT NULL DEFAULT 0,
                      log_messages TEXT NULL,
                      PRIMARY KEY (id)
                      ) ".$charsetCollate.";";

        foreach($tblSQLs as $tableSql){
            $wpdb->query($tableSql);
        }
    }

    public function createIndexesIfNotExisting(){
        $res = array();
        $res[] 	= WADA_Database::createIndexIfNotExists(
            WADA_Database::tbl_events(),
            'wada_events_sensors_idx1',
            array('sensor_id'));

        $res[] 	= WADA_Database::createIndexIfNotExists(
            WADA_Database::tbl_events(),
            'wada_events_users_idx1',
            array('user_id'));

        $res[] 	= WADA_Database::createIndexIfNotExists(
            WADA_Database::tbl_events(),
            'wada_events_objects_idx1',
            array('object_id'));

        $res[] 	= WADA_Database::createIndexIfNotExists(
            WADA_Database::tbl_event_infos(),
            'wada_event_infos_events_idx1',
            array('event_id'));

        $res[] 	= WADA_Database::createIndexIfNotExists(
            WADA_Database::tbl_event_notifications(),
            'wada_event_notifications_events_idx1',
            array('event_id'));

        $res[] 	= WADA_Database::createIndexIfNotExists(
            WADA_Database::tbl_event_notifications(),
            'wada_event_notifications_notifications_idx1',
            array('notification_id'));

        $filteredRes = array_filter($res);
        if(count($filteredRes)>0) {
            WADA_Log::installLog('createIndexesIfNotExisting res: ' . print_r($res, true));
        }else{
            WADA_Log::installLog('createIndexesIfNotExisting All indexes already there');
        }
        return $res;
    }

    public function setupStandardSettingsIfNeeded(){
        $settingsMetadata = WADA_Settings::getMetadataForAllSettings();
        //WADA_Log::installLog('setupStandardSettingsIfNeeded (all): '.print_r($settingsMetadata, true));
        foreach ($settingsMetadata as $settingId => $metadata) {
            //WADA_Log::installLog('setupStandardSettingsIfNeeded (setting id ' . $settingId . '): '.print_r($metadata, true));
            if(!WADA_Settings::doesSettingExist($settingId)){
                call_user_func(array('WADA_Settings', $metadata->setMethod), $metadata->defaultVal);
            }
        }
    }

    public function setupSensorsIfNeeded(){
        if(!class_exists('WADA_Sensor_Base')){
            require_once('Sensors/Base.php');
        }
        $sensors = array();
        $sensors[WADA_Sensor_Base::EVT_USER_REGISTRATION] = ['id' => WADA_Sensor_Base::EVT_USER_REGISTRATION,
            'severity' => 2,
            'active' => 1,
            'name' => __('Registration', 'wp-admin-audit'),
            'description' => __('Records when an account is created by a user registration.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_USER,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_USER_LOGIN] = ['id' => WADA_Sensor_Base::EVT_USER_LOGIN,
            'severity' => 1,
            'active' => 1,
            'name' => __('Login', 'wp-admin-audit'),
            'description' => __('Records when a user or admin logs into your WordPress site.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_USER,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_USER_LOGOUT] = ['id' => WADA_Sensor_Base::EVT_USER_LOGOUT,
            'severity' => 1,
            'active' => 1,
            'name' => __('Logout', 'wp-admin-audit'),
            'description' => __('Records when a user or admin logs out of your WordPress site.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_USER,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_USER_UPDATE] = ['id' => WADA_Sensor_Base::EVT_USER_UPDATE,
            'severity' => 2,
            'active' => 1,
            'name' => __('User update', 'wp-admin-audit'),
            'description' => __('Records when a user profile was edited.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_USER,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_USER_DELETE] = ['id' => WADA_Sensor_Base::EVT_USER_DELETE,
            'severity' => 3,
            'active' => 1,
            'name' => __('User deletion', 'wp-admin-audit'),
            'description' => __('Records when a user account was deleted.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_USER,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_POST_CREATE] = ['id' => WADA_Sensor_Base::EVT_POST_CREATE,
            'severity' => 2,
            'active' => 1,
            'name' => __('Post creation', 'wp-admin-audit'),
            'description' => __('Records when a post was created.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_POST,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_POST_UPDATE] = ['id' => WADA_Sensor_Base::EVT_POST_UPDATE,
            'severity' => 2,
            'active' => 1,
            'name' => __('Post update', 'wp-admin-audit'),
            'description' => __('Records when a post was updated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_POST,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_POST_DELETE] = ['id' => WADA_Sensor_Base::EVT_POST_DELETE,
            'severity' => 3,
            'active' => 1,
            'name' => __('Post deletion', 'wp-admin-audit'),
            'description' => __('Records when a post was deleted.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_POST,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_PLG_WADA_SENSOR_UPDATE] = ['id' => WADA_Sensor_Base::EVT_PLG_WADA_SENSOR_UPDATE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Sensor update (WP Admin Audit)', 'wp-admin-audit'),
            'description' => __('Records when a WP Admin Audit sensor was updated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_PLG_WADA,
            'event_category' => WADA_Sensor_Base::CAT_PLUGIN
        ];
        $sensors[WADA_Sensor_Base::EVT_USER_LOGIN_FAILED] = ['id' => WADA_Sensor_Base::EVT_USER_LOGIN_FAILED,
            'severity' => 3,
            'active' => 1,
            'name' => __('Login failed', 'wp-admin-audit'),
            'description' => __('Records when a user or admin fails to login into the site (due to incorrect credentials).', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_USER,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_USER_PASSWORD_RESET] = ['id' => WADA_Sensor_Base::EVT_USER_PASSWORD_RESET,
            'severity' => 3,
            'active' => 1,
            'name' => __('Password reset', 'wp-admin-audit'),
            'description' => __('Records when a password reset is done.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_USER,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_POST_TRASHED] = ['id' => WADA_Sensor_Base::EVT_POST_TRASHED,
            'severity' => 3,
            'active' => 1,
            'name' => __('Post trashed', 'wp-admin-audit'),
            'description' => __('Records when a post was moved into trash.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_POST,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_POST_PUBLISHED] = ['id' => WADA_Sensor_Base::EVT_POST_PUBLISHED,
            'severity' => 2,
            'active' => 1,
            'name' => __('Post published', 'wp-admin-audit'),
            'description' => __('Records when a post was published.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_POST,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_POST_UNPUBLISHED] = ['id' => WADA_Sensor_Base::EVT_POST_UNPUBLISHED,
            'severity' => 2,
            'active' => 1,
            'name' => __('Post unpublished', 'wp-admin-audit'),
            'description' => __('Records when a post was unpublished (moved out of the published status).', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_POST,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_PLG_WADA_SETTINGS_UPDATE] = ['id' => WADA_Sensor_Base::EVT_PLG_WADA_SETTINGS_UPDATE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Settings update (WP Admin Audit)', 'wp-admin-audit'),
            'description' => __('Records when the WP Admin Audit settings were updated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_PLG_WADA,
            'event_category' => WADA_Sensor_Base::CAT_PLUGIN
        ];
        /* No setup of non-core IDs:
            const EVT_PLG_PSEUDO = 16;
            const EVT_PLG_WC_PRODUCT_CREATE = 17;
            const EVT_PLG_WC_PRODUCT_UPDATE = 18;
            const EVT_PLG_WC_PRODUCT_PUBLISHED = 19;
            const EVT_PLG_WC_PRODUCT_UNPUBLISHED = 20;
            const EVT_PLG_WC_PRODUCT_TRASHED = 21;
            const EVT_PLG_WC_PRODUCT_DELETED = 22;
        */
        $sensors[WADA_Sensor_Base::EVT_PLUGIN_INSTALL] = ['id' => WADA_Sensor_Base::EVT_PLUGIN_INSTALL,
            'severity' => 4,
            'active' => 1,
            'name' => __('Plugin installation', 'wp-admin-audit'),
            'description' => __('Records when a new plugin was installed.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_PLUGIN,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_PLUGIN_DELETE] = ['id' => WADA_Sensor_Base::EVT_PLUGIN_DELETE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Plugin deletion', 'wp-admin-audit'),
            'description' => __('Records when a plugin was deleted.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_PLUGIN,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_PLUGIN_ACTIVATE] = ['id' => WADA_Sensor_Base::EVT_PLUGIN_ACTIVATE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Plugin activation', 'wp-admin-audit'),
            'description' => __('Records when a plugin was activated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_PLUGIN,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_PLUGIN_DEACTIVATE] = ['id' => WADA_Sensor_Base::EVT_PLUGIN_DEACTIVATE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Plugin deactivation', 'wp-admin-audit'),
            'description' => __('Records when a plugin was deactivated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_PLUGIN,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_PLUGIN_UPDATE] = ['id' => WADA_Sensor_Base::EVT_PLUGIN_UPDATE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Plugin update', 'wp-admin-audit'),
            'description' => __('Records when a plugin was updated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_PLUGIN,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_THEME_INSTALL] = ['id' => WADA_Sensor_Base::EVT_THEME_INSTALL,
            'severity' => 4,
            'active' => 1,
            'name' => __('Theme installation', 'wp-admin-audit'),
            'description' => __('Records when a theme was installed.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_THEME,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_THEME_DELETE] = ['id' => WADA_Sensor_Base::EVT_THEME_DELETE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Theme deletion', 'wp-admin-audit'),
            'description' => __('Records when a theme was deleted.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_THEME,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_THEME_SWITCH] = ['id' => WADA_Sensor_Base::EVT_THEME_SWITCH,
            'severity' => 4,
            'active' => 1,
            'name' => __('Theme switch', 'wp-admin-audit'),
            'description' => __('Records when the theme was switched.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_THEME,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_THEME_UPDATE] = ['id' => WADA_Sensor_Base::EVT_THEME_UPDATE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Theme update', 'wp-admin-audit'),
            'description' => __('Records when a theme was updated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_THEME,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_CORE_UPDATE] = ['id' => WADA_Sensor_Base::EVT_CORE_UPDATE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Core update', 'wp-admin-audit'),
            'description' => __('Records when WordPress was updated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_CORE,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_MEDIA_CREATE] = ['id' => WADA_Sensor_Base::EVT_MEDIA_CREATE,
            'severity' => 2,
            'active' => 1,
            'name' => __('Media creation', 'wp-admin-audit'),
            'description' => __('Records when a new media item was created.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_MEDIA,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_MEDIA_DELETE] = ['id' => WADA_Sensor_Base::EVT_MEDIA_DELETE,
            'severity' => 3,
            'active' => 1,
            'name' => __('Media deletion', 'wp-admin-audit'),
            'description' => __('Records when a media item was deleted.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_MEDIA,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_MEDIA_UPDATE] = ['id' => WADA_Sensor_Base::EVT_MEDIA_UPDATE,
            'severity' => 2,
            'active' => 1,
            'name' => __('Media update', 'wp-admin-audit'),
            'description' => __('Records when a media item was updated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_MEDIA,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_SETTING_GENERAL_UPDATE] = ['id' => WADA_Sensor_Base::EVT_SETTING_GENERAL_UPDATE,
            'severity' => 4,
            'active' => 1,
            'name' => __('General settings update', 'wp-admin-audit'),
            'description' => __('Records when the WordPress general settings were updated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_SETTING,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_SETTING_WRITING_UPDATE] = ['id' => WADA_Sensor_Base::EVT_SETTING_WRITING_UPDATE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Writing settings update', 'wp-admin-audit'),
            'description' => __('Records when the WordPress writing settings were updated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_SETTING,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_SETTING_READING_UPDATE] = ['id' => WADA_Sensor_Base::EVT_SETTING_READING_UPDATE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Reading settings update', 'wp-admin-audit'),
            'description' => __('Records when the WordPress reading settings were updated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_SETTING,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_SETTING_DISCUSSION_UPDATE] = ['id' => WADA_Sensor_Base::EVT_SETTING_DISCUSSION_UPDATE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Discussion settings update', 'wp-admin-audit'),
            'description' => __('Records when the WordPress discussion settings were updated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_SETTING,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_SETTING_MEDIA_UPDATE] = ['id' => WADA_Sensor_Base::EVT_SETTING_MEDIA_UPDATE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Media settings update', 'wp-admin-audit'),
            'description' => __('Records when the WordPress media settings were updated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_SETTING,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_SETTING_PERMALINK_UPDATE] = ['id' => WADA_Sensor_Base::EVT_SETTING_PERMALINK_UPDATE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Permalink settings update', 'wp-admin-audit'),
            'description' => __('Records when the WordPress permalink settings were updated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_SETTING,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];
        $sensors[WADA_Sensor_Base::EVT_SETTING_PRIVACY_UPDATE] = ['id' => WADA_Sensor_Base::EVT_SETTING_PRIVACY_UPDATE,
            'severity' => 4,
            'active' => 1,
            'name' => __('Privacy settings update', 'wp-admin-audit'),
            'description' => __('Records when the WordPress privacy settings were updated.', 'wp-admin-audit'),
            'event_group' => WADA_Sensor_Base::GRP_SETTING,
            'event_category' => WADA_Sensor_Base::CAT_CORE
        ];

        return self::addSensorsToDatabase($sensors);
    }

    public static function addSensorsToDatabase($sensors){
        global $wpdb;
        $sensorIds = array_column($sensors, 'id');
        $checkQuery = 'SELECT sensor_catalog.sensor_id AS id FROM (SELECT 0 AS sensor_id UNION ALL SELECT '. implode(' UNION ALL SELECT ', $sensorIds).') sensor_catalog ';
        $checkQuery .= 'LEFT JOIN '.WADA_Database::tbl_sensors().' sen ON (sensor_catalog.sensor_id = sen.id) ';
        $checkQuery .= 'WHERE sen.id IS NULL AND sensor_catalog.sensor_id > 0';
        //WADA_Log::installLog('addSensorsToDatabase: '.$checkQuery);

        $nrSensorsAdded = 0;
        $sensorToAdd = $wpdb->get_results($checkQuery, ARRAY_A);
        if(count($sensorToAdd)>0) {
            WADA_Log::installLog('addSensorsToDatabase #sensors to add: '.count($sensorToAdd));
            $sensorIdsToAdd = array_column($sensorToAdd, 'id');

            //WADA_Log::installLog('addSensorsToDatabase sensorToAdd '.print_r($sensorToAdd, true));
            //WADA_Log::installLog('addSensorsToDatabase sensorIdsToAdd: '.print_r($sensorIdsToAdd, true));

            $sensorModel = new WADA_Model_Sensor();

            foreach ($sensorIdsToAdd as $sensorId) {
                if ($sensorId > 0 && array_key_exists($sensorId, $sensors)) {
                    $newSensor = $sensors[$sensorId];
                    WADA_Log::installLog('setupSensorsIfNeeded add sensor ' . print_r($newSensor, true));
                    $res = $sensorModel->store($newSensor);
                    if ($res === false) {
                        WADA_Log::installLog('addSensorsToDatabase Failed to create sensor ' . $sensorId . ', error from model: '.$sensorModel->_last_error);
                    } else {
                        WADA_Log::installLog('addSensorsToDatabase Added sensor ' . $sensorId);
                        $nrSensorsAdded++;
                    }
                }
            }
        }else{
            WADA_Log::installLog('addSensorsToDatabase All sensors in DB, nothing new to add');
        }
        return $nrSensorsAdded;
    }

    protected function getCurrentDatabaseVersion(){
        $currentVersion = '1.1.2.0';
        return implode('.', array_slice(explode('.', $currentVersion), 0, 3)); // only extract first three version levels
    }

    public function isDatabaseUpdateNeeded(){
        $dbVersion = WADA_Settings::getDatabaseVersion('1.0.0');
        $currentVersion = $this->getCurrentDatabaseVersion();
        $updateNeeded = (version_compare($currentVersion, $dbVersion, ">") || 'false' === 'true');
        WADA_Log::installLog('isDatabaseUpdateNeeded Update needed (current: '.$currentVersion.', db version: '.$dbVersion.'): '.($updateNeeded?'yes':'no'));
        return $updateNeeded;

    }

    protected function runUpdatesAndMigrations(){
        WADA_Log::installLog('runUpdatesAndMigrations');
        $updateNeeded = $this->isDatabaseUpdateNeeded();

        if(!$updateNeeded){
            return;
        }

        $dbVersion = WADA_Settings::getDatabaseVersion('1.0.0');
        $currentVersion = $this->getCurrentDatabaseVersion();

        WADA_Log::installLog('runUpdatesAndMigrations update needed (from '.$dbVersion.' to '.$currentVersion.')');

        // **********************************************
        // DB migrations, schema updates etc go here
        // **********************************************

        if(version_compare($currentVersion, $dbVersion, ">")){
            WADA_Log::installLog('runUpdatesAndMigrations set DB version to '.$currentVersion);
             WADA_Settings::setDatabaseVersion($currentVersion); // set DB to latest version
        }

    }

    protected function scheduleEvents(){
        wp_clear_scheduled_hook( 'wp_admin_audit_maintenance' ); // remove the one set by earlier install/update
        wp_schedule_event( time() + 60, 'twicedaily', 'wp_admin_audit_maintenance' ); // (re-)add schedule
    }
}