<?php
/**
 * @copyright (C) 2021 - 2022 Holger Brandt IT Solutions
 * @license GPL2
*/

// Some CSV related code from Lee Willis
// Source https://github.com/leewillis77/WpListTableExportable/blob/master/src/WpListTableExportable.php
// Usage under GPL2

if(!defined('ABSPATH')){
    exit;
}
if ( ! class_exists( 'WP_List_Table' ) ) {
    require_once(ABSPATH . 'wp-admin/includes/class-wp-list-table.php');
}
abstract class WADA_View_BaseList extends WP_List_Table
{
    use WADA_View_BaseView;
    public $viewHeadline;
    public $totalItems;
    public $csvSeparator;
    const VIEW_IDENTIFIER = 'wada-base-list';
    const LIST_FORM_ID = 'wada-base-list-form';
    const AJAX_ACTION = '_wada_ajax_base_list';
    const NONCE_ACTION = 'wada-ajax-base-list-nonce';
    const NONCE_NAME = '_wada_ajax_base_list_nonce';

    abstract protected function getItemsQuery($searchTerm = null);

    public function __construct($args = array()) {
        parent::__construct($args);
    }

    public function get_bulk_actions(){
        return array(); // develop it if needed
    }

    public function process_bulk_action(){
        // develop it if needed
    }

    public function ajax_user_can() {
        return current_user_can( 'manage_options' );
    }

    protected function getFilterControls(){
        return array(); // override in subclass if needed
    }

    protected function getTopRightControls(){
        return array(); // override in subclass if needed
    }

    protected function getSubSections(){
        return array(); // override in subclass if needed
    }

    protected function areFiltersActive(){
        return false; // override in subclass if needed
    }

    protected function displayAfterList(){
        echo '<input type="hidden" name="nr-items" id="nr-items" value="' . esc_attr($this->totalItems) . '" />';  // override in subclass if needed
    }

    public function no_items() {
        if ( ! empty( $_REQUEST['s'] ) ) {
            $s = esc_html(wp_unslash(sanitize_text_field($_REQUEST['s'])));
            printf( __('No %s found for: %s', 'wp-admin-audit'), $this->_args['plural'], '<strong>' . $s . '</strong>' );
        } elseif ( $this->areFiltersActive() ) {
            printf( __('No %s found for filter selection', 'wp-admin-audit'), $this->_args['plural']);
        } else {
            printf( __('No %s yet', 'wp-admin-audit'), $this->_args['plural']);
        }
    }

    function column_default($item, $column_name){
        return $this->formatSearchResult(esc_html($item[$column_name]), $this->getSearchTerm());
    }

    protected function getSearchTerm(){
        return ( isset( $_REQUEST['s'] ) ) ? sanitize_text_field($_REQUEST['s']) : null;
    }

    protected function formatSearchResult($text, $searchTerm){
        if($searchTerm){
            return preg_replace('#'. preg_quote($searchTerm) .'#i', '<span class="wada-search-highlight">\\0</span>', $text);
        }
        return $text;
    }

    protected function get_hidden_columns(){
        return array(); // override me if you like in your subclass
    }

    protected function setupColumns(){
        $this->_column_headers = array( $this->get_columns(), $this->get_hidden_columns(), $this->get_sortable_columns() );
    }

    public function getDefaultOrder(){
        $sortableColumns = $this->get_sortable_columns();
        if(count($sortableColumns)>0){
            return $sortableColumns[array_keys($sortableColumns)[0]];
        }
        return array(null, null);
    }

    public function ajax_response()
    {
        WADA_Log::debug(static::VIEW_IDENTIFIER.'->ajax_response');
        check_ajax_referer(static::NONCE_ACTION, static::NONCE_NAME);

        $this->prepare_items();

        extract( $this->_args );
        extract( $this->_pagination_args, EXTR_SKIP );

        ob_start();
        if ( ! empty( $_REQUEST['no_placeholder'] ) )
            $this->display_rows();
        else
            $this->display_rows_or_placeholder();
        $rows = ob_get_clean();

        ob_start();
        $this->print_column_headers();
        $headers = ob_get_clean();

        ob_start();
        $this->pagination('top');
        $pagination_top = ob_get_clean();

        ob_start();
        $this->pagination('bottom');
        $pagination_bottom = ob_get_clean();

        $response = array( 'rows' => $rows );
        $response['pagination']['top'] = $pagination_top;
        $response['pagination']['bottom'] = $pagination_bottom;
        $response['column_headers'] = $headers;

        if ( isset( $total_items ) ) {
            $response['total_items'] = intval($total_items);
            $response['total_items_i18n'] = sprintf(_n('1 item', '%s items', $total_items), number_format_i18n($total_items));
        }

        if ( isset( $total_pages ) ) {
            $response['total_pages'] = intval($total_pages);
            $response['total_pages_i18n'] = number_format_i18n( $total_pages );
        }
        WADA_Log::debug(static::VIEW_IDENTIFIER.'->ajax_response: '.print_r($response, true));

        die( json_encode( $response ) );
    }

    function loadJavascriptActions() {
        ?>
        <script type="text/javascript">
            (function ($) {

                list = {

                    /** added method display
                     * for getting first sets of data
                     **/

                    display: function() {
                        list.init();
                    },

                    init: function () {
                        var timer;
                        var delay = 500;

                        $(
                            '.tablenav-pages a, ' +
                            '.manage-column.sortable a, ' +
                            '.manage-column.sorted a'
                        ).on('click', function (e) {
                            e.preventDefault();
                            var query = this.search.substring(1);

                            var data = {
                                paged: list.__query( query, 'paged' ) || '1',
                                order: list.__query( query, 'order' ) || 'desc',
                                orderby: list.__query( query, 'orderby' ) || 'id',
                                s: list.__query( query, 's' ) || ''
                            };
                            if(typeof getAdditionalParametersFromUrl === "function"){
                                let moreParameters = getAdditionalParametersFromUrl(query);
                                jQuery.extend(data, moreParameters);
                            }
                            list.update(data);
                        });

                        $(
                            'input[name=paged], ' +
                            'input[name=s]'
                        ).on('keyup', function (e) {
                            if (13 == e.which)
                                e.preventDefault();

                            let data = list.getQueryVariablesFromInputs();

                            window.clearTimeout(timer);
                            timer = window.setTimeout(function () {
                                list.update(data);
                            }, delay);
                        });

                        $('#<?php echo static::LIST_FORM_ID; ?>').on('submit', function(e){
                            e.preventDefault();
                            var submitButtonId = e.originalEvent.submitter.id;
                            if(submitButtonId == 'search-submit'){
                                let data = list.getQueryVariablesFromInputs();
                                list.update(data);
                            }
                        });

                        $('#per_page').on('change', function(e){
                           let data = list.getQueryVariablesFromInputs();
                           list.update(data);
                        });

                    },

                    getQueryVariablesFromInputs: function () {
                        let data = {
                            per_page: parseInt($('select[name=per_page]').val()) || '25',
                            paged: parseInt($('input[name=paged]').val()) || '1',
                            order: $('input[name=order]').val() || 'desc',
                            orderby: $('input[name=orderby]').val() || 'id',
                            s: $('input[name=s]').val() || ''
                        };
                        if(typeof getAdditionalQueryVariables === "function"){
                            let moreVariables = getAdditionalQueryVariables();
                            jQuery.extend(data, moreVariables);
                        }else{
                            // No additional query variables to consider
                            console.log('No add. query var.');
                        }
                        return data;
                    },

                    getUrlParameters: function () {
                        var url = document.location.href;
                        var qs = url.substring(url.indexOf('?') + 1).split('&');
                        for(var i = 0, result = {}; i < qs.length; i++){
                            qs[i] = qs[i].split('=');
                            result[qs[i][0]] = decodeURIComponent(qs[i][1]);
                        }
                        return result;
                    },

                    /** AJAX call
                     *
                     * Send the call and replace table parts with updated version!
                     *
                     * @param    object    data The data to pass through AJAX
                     */
                    update: function (data) {
                        $('#wada-admin-table-spinner').addClass('is-active').show();
                        $.ajax({
                            url: ajaxurl,
                            data: $.extend(
                                {
                                    <?php echo static::NONCE_NAME; ?>: $('#<?php echo static::NONCE_NAME; ?>').val(),
                                        action: '<?php echo static::AJAX_ACTION; ?>',
                                },
                                    data
                                ),
                        success: function (response) {
                            $('#wada-admin-table-spinner').removeClass('is-active').hide();
                            var response = $.parseJSON(response);

                            // Unbind previous events (will get replaced with the events on the updated DOM below)
                            $('.tablenav-pages a, .manage-column.sortable a, .manage-column.sorted a').off('click');
                            $('input[name=paged], input[name=s]').off('keyup');
                            $('#<?php echo static::LIST_FORM_ID; ?>').off('submit');
                            if(typeof unregisterAdditionalListEvents === "function"){ // unbind additional list events to have a clean slate again
                                unregisterAdditionalListEvents();
                            }

                            if (response.rows.length)
                                $('#the-list').html(response.rows);
                            if (response.column_headers.length)
                                $('thead tr, tfoot tr').html(response.column_headers);
                            if (response.pagination.bottom.length)
                                $('.tablenav.top .tablenav-pages').html($(response.pagination.top).html());
                            if (response.pagination.top.length)
                                $('.tablenav.bottom .tablenav-pages').html($(response.pagination.bottom).html());

                            if($('input#nr-items').length){
                                $('input#nr-items').val(response.total_items);
                            }

                            list.init();
                            if(typeof registerAdditionalListEvents === "function"){ // make sure we are adding back the events initiated in the overriding subclass (if needed)
                                registerAdditionalListEvents();
                            }

                            if(typeof callAfterListUpdates === "function"){ // make sure we are adding back the events initiated in the overriding subclass (if needed)
                                callAfterListUpdates();
                            }

                            let baseUrl = document.location.protocol +"//"+ document.location.hostname + document.location.pathname;
                            let urlParams = list.getUrlParameters();
                            let mergedParams = $.extend(urlParams, data);
                            let historyUrl = baseUrl+'?'+$.param(mergedParams);
                            history.pushState( null, null, historyUrl );
                        }
                    });
                    },

                    /**
                     * Filter the URL Query to extract variables
                     *
                     * @see http://css-tricks.com/snippets/javascript/get-url-variables/
                     *
                     * @param    string    query The URL query part containing the variables
                     * @param    string    variable Name of the variable we want to get
                     *
                     * @return   string|boolean The variable value if available, false else.
                     */
                    __query: function (query, variable) {

                        var vars = query.split("&");
                        for (var i = 0; i < vars.length; i++) {
                            var pair = vars[i].split("=");
                            if (pair[0] == variable)
                                return pair[1];
                        }
                        return false;
                    },
                }

                list.display();
            })(jQuery);
        </script>
        <?php
    }

    protected function performAdditionalItemPreparation(){ // override to do additional manipulations
        return $this->items;
    }


    protected function get_items_per_page( $defaultUserOption = 'edit_post_per_page', $backupDefault = 20 ) { // overriding parent method
        $userOptionName = 'per_page_'.static::VIEW_IDENTIFIER;
        $perPage = array_key_exists('per_page', $_REQUEST) ? intval($_REQUEST['per_page']) : 0; // anything in the request will be taken
        if($perPage < 1 || $perPage > 999) {
            $perPage = (int)get_user_option($userOptionName);
            if($perPage < 1 || $perPage > 999) {
                $perPage = (int)get_user_option($defaultUserOption);
                if (empty($perPage) || $perPage < 1) {
                    $perPage = $backupDefault;
                }
            }
        }
        update_user_option(get_current_user_id(), $userOptionName, $perPage);
        return intval($perPage);
    }

    public function prepare_items() {
        WADA_Log::debug(static::VIEW_IDENTIFIER.'->prepare_items');
        $this->setupColumns();
        $this->process_bulk_action();

        $searchTerm = $this->getSearchTerm();
        $perPage     = $this->get_items_per_page();
        $currentPage = $this->get_pagenum();
        $totalItems  = $this->getNrOfItems($searchTerm);

        $this->set_pagination_args( array(
            'total_items' => $totalItems, //WE have to calculate the total number of items
            'per_page'    => $perPage //WE have to determine how many items to show on a page
        ) );

        $this->totalItems = $totalItems;
        $this->items = $this->getItems($perPage, $currentPage, $searchTerm);
        $this->items = $this->performAdditionalItemPreparation();
    }

    protected function renderFilterControl($filterControl){
        switch($filterControl->type){
            case 'select':
                WADA_HtmlUtils::selectField($filterControl->field, $filterControl->label, $filterControl->value, $filterControl->selectOptions, array(), false, property_exists($filterControl, 'options') ? $filterControl->options : array());
                break;
            case 'bool':
                WADA_HtmlUtils::checkboxField($filterControl->field, $filterControl->label, $filterControl->value, property_exists($filterControl, 'options') ? $filterControl->options : array());
                break;
            case 'html':
                echo wp_kses_post($filterControl->value);
                break;
            case 'button':
                WADA_HtmlUtils::button($filterControl->name, $filterControl->label, $filterControl->iconClass, $filterControl->action, property_exists($filterControl, 'options') ? $filterControl->options : array());
                break;
            case 'button-link':
                WADA_HtmlUtils::buttonLink($filterControl->name, $filterControl->href, $filterControl->label, $filterControl->iconClass, property_exists($filterControl, 'options') ? $filterControl->options : array());
                break;
        }
    }

    protected function renderSubSections($sections){
        $html = '<ul class="subsubsub">';
        foreach($sections AS $key => $section){
            $html .= '<li class="'.esc_attr($section->class).'">';
            $page = '?page='.$section->page.($section->default ? '' : '&'.$section->param.'='.sanitize_text_field($section->paramValue));
            $currentClass = ($section->current ? 'current' : '');
            $html .= '<a href="'.esc_url($page).'" class="'.esc_attr($currentClass).'" '.($section->current ? 'aria-current="page"' : '').'>'.esc_html($section->title).' <span class="count">('.intval($section->count).')</span></a>';
            if($key < count($sections)-1){
                $html .= ' |';
            }
            $html .= '</li> ';
        }
        $html .= '</ul>';
        echo $html;
    }

    protected function display_tablenav( $which ) {
        if ( 'top' === $which ) {
            wp_nonce_field( 'bulk-' . $this->_args['plural'] );
            $subSections = $this->getSubSections();
            if($subSections && is_array($subSections) && count($subSections) > 0){
                $this->renderSubSections($subSections);
            }
        }
        ?>

        <div class="tablenav <?php echo esc_attr( $which ); ?>">
            <?php if ( $this->has_items() ) : ?>
                <div class="alignleft actions bulkactions">
                    <?php $this->bulk_actions( $which ); ?>
                </div>
            <?php
            endif;
            $this->extra_tablenav( $which );
            ?>
            <?php
            $this->pagination( $which );
            if ( 'bottom' === $which ) {
                $perPageOptions = array(5, 10, 20, 50, 100, 250, 500);
                $perPage = $this->get_items_per_page();
                $addNewOption = false;
                $addedOption = false;
                if(!in_array($perPage, $perPageOptions)){
                    $addNewOption = true;
                }
                ?>
                <div class="wada-admin-table-per-page">
                    <select name="per_page" id="per_page">
                        <?php
                        foreach($perPageOptions AS $pageOpt){
                            if($addNewOption && !$addedOption && ($perPage < $pageOpt)){
                                echo '<option value="'.intval($perPage).'" selected>'.intval($perPage).'</option>';
                                $addedOption = true;
                            }
                            echo '<option value="'.intval($pageOpt).'" '.(intval($pageOpt) === intval($perPage) ? 'selected' : '').'>'.intval($pageOpt).'</option>';
                        }
                        if($addNewOption && !$addedOption){
                            echo '<option value="'.intval($perPage).'" selected>'.intval($perPage).'</option>';
                            $addedOption = true;
                        }
                        ?>
                    </select>
                    <label for="per_page"><?php esc_html_e('entries per page', 'wp-admin-audit'); ?></label>
                </div>
                <?php
            }
            ?>
            <br class="clear" />
        </div>
        <?php
    }

    protected function extra_tablenav($which){
        if($which == "top"){
            $filterControls = $this->getFilterControls();
            if($filterControls && is_array($filterControls) && count($filterControls) > 0){
            ?>
                <div class="alignleft actions bulkactions">
                    <?php
                    foreach($filterControls AS $filterControl){
                        $this->renderFilterControl($filterControl);
                    }
                    ?>
                </div>
            <?php
            }
            $topRightControls = $this->getTopRightControls();
            if($topRightControls && is_array($topRightControls) && count($topRightControls) > 0){
                ?>
                <div class="alignright actions bulkactions">
                    <?php
                    foreach($topRightControls AS $ctrl){
                        $this->renderFilterControl($ctrl);
                    }
                    ?>
                </div>
                <?php
            }
        }
        if($which == "bottom"){
            //The code that goes after the table is there
            ?><div style="display:none;"></div><?php
        }
    }

    public function displayList() {
        ?>
        <div class="wrap">
            <h1><?php echo esc_html($this->viewHeadline); ?></h1>
            <div id="post-body" class="metabox-holder columns-2">
                <div id="post-body-content">
                    <div class="meta-box-sortables ui-sortable">
                        <form id="<?php echo static::LIST_FORM_ID; ?>" method="post" class="wada-admin-table">
                            <?php
                            $this->prepare_items();
                            $this->search_box( __( 'Search', 'wp-admin-audit' ), 'search_box' );
                            $this->display();
                            $this->displayAfterList();
                            wp_nonce_field(static::NONCE_ACTION, static::NONCE_NAME);
                            ?>
                        </form>
                    </div>
                </div>
            </div>
            <br class="clear">
        </div>
        <?php
    }

    public function getNrOfItems($searchTerm = null) {
        global $wpdb;
        $sql = $this->getItemsQuery($searchTerm);
        $sql = "SELECT COUNT(*) FROM (".$sql.") GT";
        $noItems = $wpdb->get_var( $sql );
        WADA_Log::debug('getNrOfItems: '.$noItems);
        return $noItems;
    }

    /** Override search box to get it to auto-search (and not have a submit button) */
    public function search_box( $text, $input_id ) {
        if ( empty( $_REQUEST['s'] ) && ! $this->has_items() ) {
            return;
        }
        $input_id = $input_id . '-search-input';

        if ( ! empty( $_REQUEST['orderby'] ) ) {
            $orderBy = sanitize_text_field($_REQUEST['orderby']);
            echo '<input type="hidden" name="orderby" value="' . esc_attr($orderBy) . '" />';
        }
        if ( ! empty( $_REQUEST['order'] ) ) {
            $order = sanitize_text_field($_REQUEST['order']);
            echo '<input type="hidden" name="order" value="' . esc_attr($order) . '" />';
        }
        ?>
        <p class="search-box">
            <label class="screen-reader-text" for="<?php echo esc_attr( $input_id ); ?>"><?php echo esc_html($text); ?>:</label>
            <input type="search" id="<?php echo esc_attr( $input_id ); ?>" class="wp-filter-search" name="s" value="<?php _admin_search_query(); ?>" placeholder="<?php esc_attr_e('Search list...', 'wp-admin-audit'); ?>" />
            <?php submit_button( $text, 'hide-if-js', '', false, array( 'id' => 'search-submit' ) ); ?>
        </p>
        <span id="wada-admin-table-spinner" class="spinner" style="display: none"></span>
        <?php
    }

    public function getItems($perPage, $pageNumber, $searchTerm = null){
        global $wpdb;
        $sql = $this->getItemsQuery($searchTerm);
        $orderBy = $order = null;
        if ( ! empty( $_REQUEST['orderby'] ) ) {
            $orderBy = sanitize_sql_orderby( $_REQUEST['orderby'] );
            $order = !empty( $_REQUEST['order'] ) ? sanitize_text_field( $_REQUEST['order'] ) : 'ASC';
        }elseif(method_exists($this, 'getDefaultOrder')){
                list($orderBy, $order) = $this->getDefaultOrder();
        }
        if($orderBy) {
            $order = strtoupper(trim(sanitize_text_field($order)));
            if($order !== 'ASC' AND $order !== 'DESC'){
                $order = 'ASC';
            }
            $sql .= ' ORDER BY ' . sanitize_sql_orderby($orderBy) . ' ' . $order;
        }
        if($perPage > 0) {
            $sql .= " LIMIT $perPage";
            $sql .= ' OFFSET ' . ( $pageNumber - 1 ) * $perPage;
        }
        WADA_Log::debug('getItems sql: '.$sql);
        return $wpdb->get_results( $sql, 'ARRAY_A' );

    }

    public function execute(){
        $this->displayList();
    }

    protected function getCSVColumns(){
        $columns = method_exists($this, 'get_csv_columns') ? $this->get_csv_columns() : $this->get_columns();
        $hiddenColumns = method_exists($this, 'get_hidden_csv_columns') ? $this->get_hidden_csv_columns() : $this->get_hidden_columns();
        $csvColumns = array();
        foreach($columns AS $colKey => $columnDisplayName){
            if(in_array($colKey, $hiddenColumns)){
                continue;
            }else{
                $csvColumns[$colKey] = $columnDisplayName;
            }
        }
        return $csvColumns;
    }

    protected function getCSVSeparator(){
        return ( isset( $_REQUEST['csvsep'] ) ) ? sanitize_text_field($_REQUEST['csvsep']) : ',';
    }

    /**
     * Clean up a string for use in CSV.
     *
     * NOTE: This isn't about escaping, but it tidies up a string that was originally targeted at
     * HTML output, and tries to make it CSV friendly.
     */
    protected function clean( $string ) {
        // Replace <br> with a space.
        $string = preg_replace( '#<br\s*/?>#i', ' ', $string );
        // Strip all other tags.
        $string = strip_tags( $string );
        // Decode any HTML entitites.
        $string = html_entity_decode( $string, ENT_COMPAT, 'UTF-8' );
        return $string;
    }

    protected function getCSVFilename(){
        return 'download-' . date( 'Y-m-d_His' ) . '.csv';
    }

    /**
     * Output an array using fputcsv to standard output.
     */
    protected function writeToCSV( $data ) {
        $out = fopen( 'php://output', 'w' );
        fputcsv( $out, $data,$this->csvSeparator);
        fclose( $out );
    }

    protected function prepareItemsForCSV(){
        $csvSep = $this->getCSVSeparator();
        $searchTerm = $this->getSearchTerm();
        $perPage = 0; // no limit
        $currentPage = 1; // first and only page
        $this->items = $this->getItems($perPage, $currentPage, $searchTerm);
        $this->items = $this->performAdditionalItemPreparation();
        $this->csvSeparator = $csvSep;
        WADA_Log::debug(static::VIEW_IDENTIFIER.'->prepareItemsForCSV csvSep: '.$this->csvSeparator);
        //WADA_Log::debug(static::VIEW_IDENTIFIER.'->prepareItemsForCSV items: '.print_r($this->items, true));
    }

    protected function printCSVHeader(){
        $filename = $this->getCSVFilename();
        $csvColumns = $this->getCSVColumns();
        $displayNames = array_values($csvColumns);
        WADA_Log::debug(static::VIEW_IDENTIFIER.'->printCSVHeader filename: '.$filename);
        WADA_Log::debug(static::VIEW_IDENTIFIER.'->printCSVHeader csvColumns: '.print_r($csvColumns, true));
        WADA_Log::debug(static::VIEW_IDENTIFIER.'->printCSVHeader displayNames: '.print_r($displayNames, true));
        header( 'Content-Type: text/csv; charset=utf-8' );
        header( 'Content-Disposition: attachment; filename="' . $filename . '"' );
        $this->writeToCSV($displayNames);
    }

    protected function printCSVBody(){
        $csvColumns = $this->getCSVColumns();
        foreach ($this->items as $item) {
            $row = array();
            foreach ($csvColumns as $colKey => $displayName) {
                $cell = null;
                if (method_exists($this, 'column_csv_' . $colKey)) {
                    // WADA_Log::debug('printCSVBody for '.$colKey.': custom csv method');
                    $cell = call_user_func(array($this, 'column_csv_' . $colKey), $item);
                } elseif (method_exists($this, 'column_' . $colKey)) {
                    // WADA_Log::debug('printCSVBody for '.$colKey.': custom regular method');
                    $cell = $this->clean(
                        call_user_func(array($this, 'column_' . $colKey), $item)
                    );
                } else {
                    // WADA_Log::debug('printCSVBody for '.$colKey.': default method');
                    $cell = $this->clean(
                        $this->column_default($item, $colKey)
                    );
                }
                // WADA_Log::debug('printCSVBody for '.$colKey.' cell content: '.$cell);
                $row[] = $cell;
            }
            $this->writeToCSV($row);
        }
    }

    public function csvDownload($dieAfterDownload = true){

        /*  */
    }

}