<?php
/**
 * @copyright (C) 2021 - 2022 Holger Brandt IT Solutions
 * @license GPL2
*/

if(!defined('ABSPATH')){
    exit;
}

class WADA_View_Events extends WADA_View_BaseList
{
    const VIEW_IDENTIFIER = 'wada-events';
    const LIST_FORM_ID = 'wada-event-list-form';
    const AJAX_ACTION = '_wada_ajax_events_list';
    const NONCE_ACTION = 'wada-ajax-events-list-nonce';
    const NONCE_NAME = '_wada_ajax_events_list_nonce';

    public function __construct() {
        $this->viewHeadline =  __('Event Log', 'wp-admin-audit');
        parent::__construct( array(
            'singular' => __( 'Event', 'wp-admin-audit' ), //singular name of the grouped records
            'plural'   => __( 'Events', 'wp-admin-audit' ), //plural name of the grouped records
            'ajax'     => true // does this table support ajax?
        ) );
        add_action('admin_footer', array($this, 'loadJavascriptActions'));
        WADA_ScriptUtils::loadCsvSepUtil();
    }

    function get_columns() {
        $colArray = array(
            'id'    => __( 'ID', 'wp-admin-audit' ),
            'severity'    => __( 'Severity', 'wp-admin-audit' ),
            'occurred_on'    => __( 'Date', 'wp-admin-audit' ),
            'user_name'    => __( 'User', 'wp-admin-audit' ),
            'sensor_name' => __( 'Event Type', 'wp-admin-audit' )
        );
        $searchTerm = $this->getSearchTerm();
        if($searchTerm){
            $colArray['search_res'] = __( 'Search result', 'wp-admin-audit' );
        }
        return $colArray;
    }

    public function get_sortable_columns() {
        return array(
            'id' => array('id', 'desc'),
            'severity' => array('severity', 'desc'),
            'occurred_on' => array('occurred_on', 'desc'),
            'user_name' => array('user_name', 'asc'),
            'sensor_name' => array('sensor_name', 'asc')
        );
    }

    function column_id($item) {
        //WADA_Log::debug('column_id for item '.print_r($item, true));
        $nonce = wp_create_nonce( 'wada_events_view' );
        return sprintf(
            '<a href="?page=wp-admin-audit-events&subpage=%s&amp;sid=%s&amp;_wpnonce=%s"><strong>%s</strong></a>',
            'details',
            absint( $item['id'] ),
            $nonce,
            '#'.absint($item['id']) . ' '.esc_html($item['sensor_name'])
        );
    }

    function column_severity($item){
        return esc_html($item['severity_text']);
    }

    function column_occurred_on($item){
        return WADA_DateUtils::formatUTCasDatetimeForWP($item['occurred_on']);
    }

    protected function getSensorFilterFromRequest($default = null){
        $sensorId = $default;
        $allowedOptions = array_keys($this->getSensorOptions());
        if(array_key_exists('sensor', $_GET)){
            $reqValue = intval($_GET['sensor']);
            if($reqValue > 0 && in_array($reqValue, $allowedOptions)){
                $sensorId = $reqValue;
            }
        }
        return $sensorId;
    }

    protected function getDayLimitFilterFromRequest($default = 0){
        $dayLimit = $default;
        $allowedOptions = $this->getDayLimitOptions();
        if(array_key_exists('days', $_GET)){
            $reqValue = intval($_GET['days']);
            if($reqValue > 0 && in_array($reqValue, $allowedOptions)){
                $dayLimit = $reqValue;
            }
        }
        return intval($dayLimit);
    }

    protected function getSensorOptions(){
        global $wpdb;
        $sensorOptions = array();
        $sensorOptions[''] = __('All event types', 'wp-admin-audit');
        $sql= "SELECT sen.id, sen.name, sen.event_group, sen.event_category "
            ."FROM ".WADA_Database::tbl_sensors() ." sen "
            ."WHERE sen.id <> '".WADA_Sensor_Base::EVT_PLG_PSEUDO."'";
        $sensors = $wpdb->get_results($sql);

        foreach($sensors AS $sensor){
            $opt = new stdClass();
            $opt->id = $sensor->id;
            $opt->name = $sensor->name;
            $opt->cat = WADA_Model_Sensor::getEventGroupName($sensor->event_group);
            $sensorOptions[$sensor->id] = $opt;
        }
        return $sensorOptions;
    }

    protected function getDayLimitOptions(){
        return array(0, 7, 30, 90);
    }

    function displayAfterList(){

        /*  */

        parent::displayAfterList();
    }

    protected function getFilterControls(){
        $filterControls = array();

        $eventGroupFilter = new stdClass();
        $eventGroupFilter->type = 'select';
        $eventGroupFilter->value = $this->getSensorFilterFromRequest();
        $eventGroupFilter->field = 'sensor';
        $eventGroupFilter->label = null; // no need for label
        $eventGroupFilter->selectOptions = $this->getSensorOptions();
        $eventGroupFilter->options = array('use_optgroup'=>true);
        $filterControls[] = $eventGroupFilter;

        /*  */


        // WADA_Log::debug('getFilterControls: '.print_r($filterControls, true));

        return $filterControls;
    }


    protected function performAdditionalItemPreparation(){
        $severityLevels = WADA_Model_Sensor::getSeverityLevels();
        foreach($this->items AS $key => $item){
            $this->items[$key]['severity_text'] = $severityLevels[$item['severity']];
        }
        return $this->items;
    }

    protected function getItemsQuery($searchTerm = null){
        $sensorId = $this->getSensorFilterFromRequest();
        $dayLimit = $this->getDayLimitFilterFromRequest();
        $sql= "SELECT evt.id, evt.occurred_on, evt.sensor_id, "
            ."evt.user_id, evt.user_name, evt.user_email, "
            ."evt.object_type, evt.object_id, evt.source_ip, "
            ."sen.name as sensor_name, sen.severity, sen.event_group, sen.event_category "
            .($searchTerm ? ", ei_search_res.search_res as search_res " : ", NULL AS search_res ")
            ."FROM ".WADA_Database::tbl_events() . " evt "
            ."LEFT JOIN ".WADA_Database::tbl_sensors() ." sen ON (evt.sensor_id = sen.id)";
        if($searchTerm) {
            $sql .=
            "LEFT JOIN ("
                . "SELECT event_id, GROUP_CONCAT(DISTINCT search_res ORDER BY search_res SEPARATOR ', ') AS search_res "
                . "FROM ( "
                . "	SELECT event_id, info_value as search_res FROM " . WADA_Database::tbl_event_infos() . " ei1 WHERE info_value LIKE '%" . $searchTerm . "%' "
                . "	UNION SELECT event_id, info_value as search_res FROM " . WADA_Database::tbl_event_infos() . " ei2 WHERE prior_value LIKE '%" . $searchTerm . "%' "
                . ") ei_res  "
                . "GROUP BY event_id "
            . ") ei_search_res ON (evt.id = ei_search_res.event_id)";

            $fieldsToSearchIn = array('evt.user_name', 'evt.user_email', 'evt.source_ip', 'sen.name');
            $sep = " LIKE '%".$searchTerm."%') OR (";
            $whereCond = implode($sep, $fieldsToSearchIn);
            $whereCond .= " LIKE '%".$searchTerm."%')";
            $whereCond .= " OR ei_search_res.event_id IS NOT NULL )";
            if($sensorId && $sensorId > 0){
                $whereCond .= ' AND (evt.sensor_id = ' . intval($sensorId) .')';
            }
            if($dayLimit > 0){
                WADA_Log::debug('Filter with dayLimit: '.$dayLimit);
                $whereCond .= ' AND (evt.occurred_on >= DATE(NOW() - INTERVAL '.intval($dayLimit).' DAY))';
            }
            $sql .= ' WHERE ( ('.$whereCond;
        }else{
            $whereCond = array();
            if($sensorId && $sensorId > 0){
                $whereCond[] = 'evt.sensor_id = ' . intval($sensorId);
            }
            if($dayLimit > 0){
                WADA_Log::debug('Filter with dayLimit: '.$dayLimit);
                $whereCond[] = '(evt.occurred_on >= DATE(NOW() - INTERVAL '.intval($dayLimit).' DAY))';
            }
            if(count($whereCond) > 0) {
                $sql .= (' WHERE ' . implode(' AND ', $whereCond));
            }
        }
        return $sql;
    }


    function loadJavascriptActions(){ // override with extending functionality ?>
        <script type="text/javascript">
            function getAdditionalQueryVariables(){
                //console.log('[Events] getAdditionalQueryVariables: ');
                let data = {
                    sensor: jQuery('select[name=sensor]').val() || ''
                };
                //console.log(data);
                return data;
            }
            function getAdditionalParametersFromUrl(query){
                //console.log('[Events] getAdditionalParametersFromUrl: ');
                //console.log(query);
                let data = {
                    sensor: list.__query( query, 'sensor' ) || '',
                };
                //console.log(data);
                return data;
            }
            function registerOneTimeEvents(){
                jQuery('.csv-export-go').on('click', function (e) {
                    jQuery('#csv-export-msg').html('').removeClass('wada-error');
                    jQuery(this).children('.wada-button-text').hide();
                    jQuery(this).children('.spinner').addClass('is-active').show();
                    jQuery('.csv-export-go').prop('disabled', true);
                    let dayLimit = parseInt(jQuery(this).data('day-limit'));
                    let csvSep = getCsvSeparator();
                    startCsvExport(dayLimit, csvSep);
                });
            }
            function registerAdditionalListEvents(){
                jQuery('#sensor').on('change', function (e) {
                    let data = list.getQueryVariablesFromInputs();
                    list.update(data);
                });
            }
            function unregisterAdditionalListEvents(){
                jQuery('#sensor').off('change');
            }
            function callAfterListUpdates(){
                jQuery('#csv-nr-items').html(parseInt(jQuery('input#nr-items').val()));
            }
            function startCsvExport(dayLimit, csvSep){
                console.log(dayLimit);
                let data = list.getQueryVariablesFromInputs();
                jQuery.extend(data, {
                    days: dayLimit,
                    csvsep: csvSep
                });
                console.log(data);
                jQuery.ajax({
                    url: ajaxurl,
                    data: jQuery.extend({
                        <?php echo static::NONCE_NAME; ?>: jQuery('#<?php echo static::NONCE_NAME; ?>').val(),
                        action: '_wada_ajax_events_csv_export'
                    }, data),
                    success: function(data) {
                        jQuery('.csv-export-go .wada-button-text').show();
                        jQuery('.csv-export-go .spinner').removeClass('is-active').hide();
                        jQuery('.csv-export-go').prop('disabled', false)
                        tb_remove(); // close modal

                        // Make CSV downloadable
                        var downloadLink = document.createElement("a");
                        var fileData = ['\ufeff'+data];

                        var blobObject = new Blob(fileData,{
                            type: "text/csv;charset=utf-8;"
                        });

                        var url = URL.createObjectURL(blobObject);
                        downloadLink.href = url;
                        let currDate = new Date();
                        downloadLink.download = "events-"+currDate.toISOString().split('T')[0]+"-"+currDate.getHours()+currDate.getMinutes()+currDate.getSeconds()+".csv";

                        // Actually download CSV
                        document.body.appendChild(downloadLink);
                        downloadLink.click();
                        document.body.removeChild(downloadLink);
                    },
                    error: function(jqXHR, textStatus, errorThrown) {
                        jQuery('.csv-export-go .wada-button-text').show();
                        jQuery('.csv-export-go .spinner').removeClass('is-active').hide();
                        jQuery('.csv-export-go').prop('disabled', false)
                        jQuery('#csv-export-msg').html('<?php echo esc_js(__('Error while exporting. Please try again later.', 'wp-admin-audit')); ?>').addClass('wada-error');
                    }
                });
            }
            (function ($) {
                registerAdditionalListEvents();
                registerOneTimeEvents();
            })(jQuery);
        </script>
        <?php
        parent::loadJavascriptActions(); // make sure base class also loads JS
    }

    function getCSVFilename(){ // override parent
        return 'events-' . date( 'Y-m-d_His' ) . '.csv';
    }

    function csvExportAjaxResponse(){
        WADA_Log::debug('csvExportAjaxResponse');
        WADA_Log::debug('csvExportAjaxResponse GET: '.print_r($_GET, true));
        check_ajax_referer(static::NONCE_ACTION, static::NONCE_NAME);

        $this->csvDownload(true);
    }

}