<?php
/**
 * @copyright (C) 2021 - 2022 Holger Brandt IT Solutions
 * @license GPL2
*/

if(!defined('ABSPATH')){
    exit;
}

class WADA_Layout_EventDetailsBase implements WADA_Layout_LayoutInterface
{
    const LAYOUT_IDENTIFIER = 'wada-layout-event-details';
    public $event;

    public function __construct($event = null){
        if($event){
            $this->event = $event;
        }
    }

    protected function extractEventInfoFromArray($infos, $key, $default=null){
        $res = array_search($key, array_column($infos, 'info_key'));
        if($res === false){
            return $default;
        }
        return $infos[$res];
    }

    protected function extractValueFromInfoArray($infos, $key, $default=null){
        $infoObj = $this->extractEventInfoFromArray($infos, $key, null);
        if($infoObj){
            return ((property_exists($infoObj, 'info_value')) ? $infoObj->info_value : $default);
        }
        return $default;
    }

    protected function extractPriorValueFromInfoArray($infos, $key, $default=null){
        $infoObj = $this->extractEventInfoFromArray($infos, $key, null);
        if($infoObj){
            return ((property_exists($infoObj, 'prior_value')) ? $infoObj->prior_value : $default);
        }
        return $default;
    }

    protected function renderDefaultEventInfosRow($field, $infoValue, $priorValue, $cssClassCurrent = '', $cssClassPrior = '', $titleCurrent = '', $titlePrior = ''){
        $innerHtml = '<tr>';
        $innerHtml .= '<td class="label">' . esc_html($field) . '</td>';
        $innerHtml .= '<td class="value curr-value ' . $cssClassCurrent . '" title="'. esc_html($titleCurrent).'">' . esc_html($infoValue) . '</td>';
        $innerHtml .= '<td class="value prior-value ' . $cssClassPrior . '" title="'. esc_html($titlePrior).'">' . esc_html($priorValue) . '</td>';
        $innerHtml .= '</tr>';
        return $innerHtml;
    }

    protected function renderDefaultEventInfosRowWithSingleValue($field, $infoValue, $cssClassCurrent = '', $titleCurrent = ''){
        $innerHtml = '<tr>';
        $innerHtml .= '<td class="label">' . esc_html($field) . '</td>';
        $innerHtml .= '<td class="value curr-value ' . $cssClassCurrent . '" title="'. esc_html($titleCurrent).'">' . esc_html($infoValue) . '</td>';
        $innerHtml .= '</tr>';
        return $innerHtml;
    }

    protected function skipLine($field, $infoValue, $priorValue){
        WADA_Log::debug('skipLine!');
        return ''; // told ya!
    }

    protected function renderDeletedLines($field, $infoValue, $priorValue){
        $cssClassCurrent = 'wada-del';
        $field = str_replace('DEL_DATA_', '', $field);
        $priorValue = $infoValue;
        $infoValue = __('(deleted)', 'wp-admin-audit');
        return $this->renderDefaultEventInfosRow($field, $infoValue, $priorValue, $cssClassCurrent);
    }

    protected function renderBooleanValueLine($field, $infoValue, $priorValue){
        $currTitle = $infoValue;
        $priorTitle = $priorValue;
        if($infoValue === '0' || $infoValue === '' || is_null($infoValue)){
            $infoValue = __('No', 'wp-admin-audit');
        }elseif($infoValue === '1'){
            $infoValue = __('Yes', 'wp-admin-audit');
        }
        if($priorValue === '0' || $priorValue === '' || is_null($priorValue)){
            $priorValue = __('No', 'wp-admin-audit');
        }elseif($priorValue === '1'){
            $priorValue = __('Yes', 'wp-admin-audit');
        }
        return $this->renderDefaultEventInfosRow($field, $infoValue, $priorValue, '', '', $currTitle, $priorTitle);
    }

    protected function renderTitleAndDefaultEventInfos($title, $subtitle, $specialInfoKeys = array(), $additionalParams = array()){
        WADA_Log::debug('renderTitleAndDefaultEventInfos');
        $html = '<h3>'.$title.'</h3><h4>'.$subtitle.'</h4>';

        if(!$this->event->infos || count($this->event->infos) == 0){
            $innerHtml = '';
        }else {
            $innerHtml = $this->renderDefaultEventInfosTable($specialInfoKeys, $additionalParams);
        }

        echo wp_kses_post($html.$innerHtml);
    }

    /**
     * @param array $specialInfoKeys
     * Example: array(
     * *            array( 'sensor_id' => 12, 'info_key'           => 'POST_STATUS_TRANSITION', 'callback' => ($this, 'skipLine') ),
     *              array( 'sensor_id' => 5,  'info_key_prefix'    => 'DEL_DATA_',              'callback' => ($this, 'renderDeletedLines') ),
     * @param array $additionalParams
     * Example: array(
     *              'ONLY_SINGLE_VALUE'
     * )
     * @return string
     */
    protected function renderDefaultEventInfosTable($specialInfoKeys = array(), $additionalParams = array()){
        $innerHtml = '';
        $onlySingleValue = in_array('ONLY_SINGLE_VALUE', $additionalParams);
        if($this->event && $this->event->infos){
            $innerHtml = '<table class="data">';
            $innerHtml .= '<tbody>';
            $innerHtml .= '<tr>';
            $innerHtml .= '<th class="label">' . esc_html(__('Field', 'wp-admin-audit')) . '</th>';
            if($onlySingleValue){
                $innerHtml .= '<th class="label">' . esc_html(__('Value', 'wp-admin-audit')) . '</th>';
            }else {
                $innerHtml .= '<th class="label">' . esc_html(__('New value', 'wp-admin-audit')) . '</th>';
                $innerHtml .= '<th class="label">' . esc_html(__('Prior value', 'wp-admin-audit')) . '</th>';
            }
            $innerHtml .= '</tr>';
            foreach ($this->event->infos as $info) {
                $renderedInfo = false;
                foreach($specialInfoKeys AS $specialInfoKey){
                    $sensorIdMatch = (!array_key_exists('sensor_id', $specialInfoKey) || ($specialInfoKey['sensor_id'] === $this->event->sensor_id));
                    $infoKeyMatch = (array_key_exists('info_key', $specialInfoKey) && $specialInfoKey['info_key'] === $info->info_key);
                    $infoKeyPrefixMatch = (array_key_exists('info_key_prefix', $specialInfoKey) && (strpos($info->info_key , $specialInfoKey['info_key_prefix']) === 0));

                    if($sensorIdMatch && ($infoKeyMatch || $infoKeyPrefixMatch)){

                        if($infoKeyMatch){
                            $info->info_key = (array_key_exists('info_key_label', $specialInfoKey)) ? $specialInfoKey['info_key_label'] : $info->info_key;
                        }

                        if(array_key_exists('callback', $specialInfoKey) && is_callable($specialInfoKey['callback'])){
                            WADA_Log::debug('going to execute: '.print_r($specialInfoKey['callback'], true));
                            $innerHtml .= call_user_func($specialInfoKey['callback'], $info->info_key, $info->info_value, $info->prior_value);
                            $renderedInfo = true; // make sure we do not execute the default method below
                            break; // no need to search on
                        }
                    }
                }
                if(!$renderedInfo) {
                    if($onlySingleValue){
                        $innerHtml .= $this->renderDefaultEventInfosRowWithSingleValue($info->info_key, $info->info_value);
                    }else{
                        $innerHtml .= $this->renderDefaultEventInfosRow($info->info_key, $info->info_value, $info->prior_value);
                    }
                }
            }
            $innerHtml .= '</tbody>';
            $innerHtml .= '</table>';
        }
        return $innerHtml;
    }

    public function display($returnAsHtml = false){ // overwritten in the subclasses to show a less generic layout
        if($returnAsHtml){
            ob_start();
        }
        echo wp_kses_post($this->renderDefaultEventInfosTable());
        if ($returnAsHtml) {
            $html = ob_get_contents();
            ob_end_clean();
            return $html;
        }
    }

    /**
     * @param stdClass $event
     * @return string
     */
    public static function getEventObjectDetailsTable($event){
        $header = $innerHtml = '';
        if(!$event->object_type){
            return '';
        }
        switch ($event->object_type) {
            case WADA_Sensor_Base::OBJ_TYPE_CORE_USER:
                $user = get_userdata(absint($event->object_id));
                $header = __('User subject', 'wp-admin-audit');
                $innerHtml = (new WADA_Layout_UserOutline($user, $event->object_id))->display(true);
                break;
            case WADA_Sensor_Base::OBJ_TYPE_CORE_POST:
                $post = get_post(absint($event->object_id));
                $header = __('Post subject', 'wp-admin-audit');
                $innerHtml = (new WADA_Layout_PostOutline($post, $event->object_id))->display(true);
                break;
            case WADA_Sensor_Base::OBJ_TYPE_PLG_WADA_SENSOR:
                $header = __('Sensor', 'wp-admin-audit');
                $model = new WADA_Model_Sensor($event->object_id);
                $innerHtml = esc_html('#'.$event->object_id.' '.$model->_data->name);
                break;
            default:
                $header = __('Object data', 'wp-admin-audit');
                $innerHtml = '<table class="data">';
                $innerHtml .= '<tbody>';
                $innerHtml .= '<tr>';
                $innerHtml .= '<td class="label">'.esc_html($event->object_type).'</td>';
                $innerHtml .= '<td class="value">'.intval($event->object_id).'</td>';
                $innerHtml .= '</tr>';
                $innerHtml .= '</tbody>';
                $innerHtml .= '</table>';
        }
        $html = '<h3>'.$header .'</h3>';
        $html .= '<div class="'.self::LAYOUT_IDENTIFIER.' wada-event-object-details-tbl">';
        $html .= $innerHtml;
        $html .= '</div>';
        return $html;
    }

    /**
     * @param stdClass $event
     * @return WADA_Layout_LayoutInterface
     */
    public static function getEventDetailsLayout($event){
        $layout = null;
        switch ($event->sensor_id) {
            case WADA_Sensor_Base::EVT_CORE_UPDATE:
                $layout = new WADA_Layout_CoreEvent($event);
                break;
            case WADA_Sensor_Base::EVT_MEDIA_CREATE:
            case WADA_Sensor_Base::EVT_MEDIA_UPDATE:
            case WADA_Sensor_Base::EVT_MEDIA_DELETE:
                $layout = new WADA_Layout_MediaEvent($event);
            break;
            case WADA_Sensor_Base::EVT_POST_CREATE:
            case WADA_Sensor_Base::EVT_POST_UPDATE:
            case WADA_Sensor_Base::EVT_POST_DELETE:
            case WADA_Sensor_Base::EVT_POST_TRASHED:
            case WADA_Sensor_Base::EVT_POST_PUBLISHED:
            case WADA_Sensor_Base::EVT_POST_UNPUBLISHED:
                $layout = new WADA_Layout_PostEvent($event);
                break;
            case WADA_Sensor_Base::EVT_SETTING_GENERAL_UPDATE:
            case WADA_Sensor_Base::EVT_SETTING_WRITING_UPDATE:
            case WADA_Sensor_Base::EVT_SETTING_READING_UPDATE:
            case WADA_Sensor_Base::EVT_SETTING_DISCUSSION_UPDATE:
            case WADA_Sensor_Base::EVT_SETTING_MEDIA_UPDATE:
            case WADA_Sensor_Base::EVT_SETTING_PERMALINK_UPDATE:
            case WADA_Sensor_Base::EVT_SETTING_PRIVACY_UPDATE:
                $layout = new WADA_Layout_SettingEvent($event);
                break;
            case WADA_Sensor_Base::EVT_USER_REGISTRATION:
            case WADA_Sensor_Base::EVT_USER_LOGIN:
            case WADA_Sensor_Base::EVT_USER_LOGIN_FAILED:
            case WADA_Sensor_Base::EVT_USER_LOGOUT:
            case WADA_Sensor_Base::EVT_USER_UPDATE:
            case WADA_Sensor_Base::EVT_USER_DELETE:
            case WADA_Sensor_Base::EVT_USER_PASSWORD_RESET:
                $layout = new WADA_Layout_UserEvent($event);
                break;
            case WADA_Sensor_Base::EVT_PLUGIN_INSTALL:
            case WADA_Sensor_Base::EVT_PLUGIN_DELETE:
            case WADA_Sensor_Base::EVT_PLUGIN_ACTIVATE:
            case WADA_Sensor_Base::EVT_PLUGIN_DEACTIVATE:
            case WADA_Sensor_Base::EVT_PLUGIN_UPDATE:
                $layout = new WADA_Layout_PluginEvent($event);
                break;
            case WADA_Sensor_Base::EVT_PLG_WADA_SENSOR_UPDATE:
            case WADA_Sensor_Base::EVT_PLG_WADA_SETTINGS_UPDATE:
                $layout = new WADA_Layout_WADAEvent($event);
                break;
            case WADA_Sensor_Base::EVT_THEME_INSTALL:
            case WADA_Sensor_Base::EVT_THEME_DELETE:
            case WADA_Sensor_Base::EVT_THEME_SWITCH:
            case WADA_Sensor_Base::EVT_THEME_UPDATE:
                $layout = new WADA_Layout_ThemeEvent($event);
                break;
            default:
                $layout = new self($event);
        }
        return $layout;
    }
}