<?php
/**
 * @copyright (C) 2021 - 2022 Holger Brandt IT Solutions
 * @license GPL2
*/

if(!defined('ABSPATH')){
    exit;
}

class WADA_View_Sensors extends WADA_View_BaseList
{
    const VIEW_IDENTIFIER = 'wada-sensors';
    const LIST_FORM_ID = 'wada-sensor-list-form';
    const AJAX_ACTION = '_wada_ajax_sensors_list';
    const NONCE_ACTION = 'wada-ajax-sensors-list-nonce';
    const NONCE_NAME = '_wada_ajax_sensors_list_nonce';

    public function __construct() {
        $this->viewHeadline =  __('Sensors', 'wp-admin-audit');
        parent::__construct( array(
            'singular' => __( 'Sensor', 'wp-admin-audit' ), //singular name of the grouped records
            'plural'   => __( 'Sensors', 'wp-admin-audit' ), //plural name of the grouped records
            'ajax'     => true // does this table support ajax?
        ) );
        add_action('admin_footer', array($this, 'loadJavascriptActions'));
    }

    function get_columns() {
        $colArray = array(
            'id'    => __( 'ID', 'wp-admin-audit' ),
            'active'    => __( 'Active', 'wp-admin-audit' ),
            'severity'    => __( 'Severity', 'wp-admin-audit' ),
            'event_group'    => __( 'Event Group', 'wp-admin-audit' ),
            'event_category'    => __( 'Event Category', 'wp-admin-audit' )
        );
        $searchTerm = $this->getSearchTerm();
        if($searchTerm){
            $colArray['search_res'] = __( 'Search result', 'wp-admin-audit' );
        }
        return $colArray;
    }

    public function get_sortable_columns() {
        return array(
            'id' => array('id', 'asc'),
            'severity' => array('severity', 'asc'),
            'active' => array('active', 'asc'),
            'event_group' => array('event_group', 'asc'),
            'event_category' => array('event_category', 'asc')
        );
    }

    function column_id($item) {
        WADA_Log::debug('column_id for item '.print_r($item, true));
        $nonce = wp_create_nonce( 'wada_sensors_view' );
        return sprintf(
            '<a href="?page=wp-admin-audit-sensors&subpage=%s&amp;sid=%s&amp;_wpnonce=%s"><strong>%s</strong></a>',
            'sensor',
            absint( $item['id'] ),
            $nonce,
            '#'.absint($item['id']) . ' '.$this->formatSearchResult(esc_html($item['name']), $this->getSearchTerm())
        );
    }

    function column_active($item){
        $disabled = $title = '';
        if($item['extension_id'] && $item['extension_active'] == 0 && $item['active'] == 0){
            $disabled = ' disabled="disabled" ';
            $title = esc_attr__(sprintf(__('The extension %s is no longer active, therefore the sensor cannot be activated', 'wp-admin-audit'), $item['extension_name']));
        }
        return '<input type="checkbox" class="wada-ui-toggle sensor-status-toggle" '
        .'id="active'.absint($item['id']).'" name="active'.absint($item['id']).'" value="1" '
        .$disabled.($item['active'] == 1 ? 'checked' : '').' title="'.$title.'">';
    }

    function column_severity($item){
        return esc_html($item['severity'] . ' (' . $item['severity_text'].')');
    }

    function column_event_category($item){
        $eventCategories = WADA_Model_Sensor::getEventCategories();
        return esc_html((array_key_exists($item['event_category'], $eventCategories) ? $eventCategories[$item['event_category']] : $item['event_category']));
    }

    function column_event_group($item){
        return esc_html($item['event_group_name']);
    }

    function toggleActionAjaxResponse(){
        WADA_Log::debug(static::VIEW_IDENTIFIER.'->toggleActionAjaxResponse');
        check_ajax_referer(static::NONCE_ACTION, static::NONCE_NAME);

        $id = isset( $_REQUEST['id'] ) ? absint( $_REQUEST['id'] ) : 0;
        $active = !isset($_REQUEST['active']) || $_REQUEST['active'] == 'true';
        if($id){
            $res = WADA_Model_Sensor::setActiveStatus($id, $active);
            $response = array('success' => $res);
        }else{
            $response = array('success' => false, 'error' => 'No ID found');
        }

        WADA_Log::debug(static::VIEW_IDENTIFIER.'->toggleActionAjaxResponse: '.print_r($response, true));

        die( json_encode( $response ) );
    }

    protected function areFiltersActive(){
        $active = $this->getActiveFilterFromRequest();
        if($active >= 0){
            return true;
        }
        return false;
    }

    protected function getActiveFilterFromRequest($default = -1){
        $active = $default;
        if(array_key_exists('active', $_GET)){
            if(trim(strval($_GET['active'])) === ''){
                return $default;
            }
            $active = intval($_GET['active']);
            if($active < -1){
                $active = -1;
            }
            if($active > 1){
                $active = 1;
            }
        }
        return $active;
    }

    protected function getEventGroupFilterFromRequest($default = null){
        $eventGroup = $default;
        $allowedOptions = array_keys($this->getEventGroupOptions());
        if(array_key_exists('event_group', $_GET)){
            $reqValue = sanitize_text_field($_GET['event_group']);
            if(in_array($reqValue, $allowedOptions)){
                $eventGroup = $reqValue;
            }
        }
        return $eventGroup;
    }

    protected function performAdditionalItemPreparation(){
        $severityLevels = WADA_Model_Sensor::getSeverityLevels();
        foreach($this->items AS $key => $item){
            $this->items[$key]['severity_text'] = $severityLevels[$item['severity']];
            $this->items[$key]['event_group_name'] = WADA_Model_Sensor::getEventGroupName($this->items[$key]['event_group']);
        }
        return $this->items;
    }

    protected function getItemsQuery($searchTerm = null){
        $active = $this->getActiveFilterFromRequest();
        $eventGroup = $this->getEventGroupFilterFromRequest();
        $sql = "SELECT sen.id, sen.name, sen.description, "
            ."sen.severity, sen.active, sen.event_group, sen.event_category, "
            ."sen.extension_id, ext.active as extension_active, ext.name as extension_name, "
            ."ext.plugin_folder as extension_plugin_folder, ext.ext_api_version as extension_api_version"
            .($searchTerm ? ", sen_search_res.search_res as search_res " : ", NULL AS search_res ")
            ."FROM ".WADA_Database::tbl_sensors() . " sen "
            ."LEFT JOIN ".WADA_Database::tbl_extensions() . " ext ON (sen.extension_id = ext.id) ";
        if($searchTerm) {
            $sql .=
                "LEFT JOIN ("
                . "SELECT id, description AS search_res "
                . "FROM " . WADA_Database::tbl_sensors() . " ei1 WHERE description LIKE '%" . $searchTerm . "%' "
                . ") sen_search_res ON (sen.id = sen_search_res.id)";
            $fieldsToSearchIn = array('sen.name', 'sen.event_group', 'sen.event_category');
            $sep = " LIKE '%".$searchTerm."%') OR (";
            $whereCond = implode($sep, $fieldsToSearchIn);
            $whereCond .= " LIKE '%".$searchTerm."%') )";
            if($active >= 0){
                $whereCond .= ' AND (sen.active = ' . intval($active) .')';
            }
            if($eventGroup){
                $whereCond .= ' AND (sen.event_group = \''.$eventGroup.'\')';
            }
            $sql .= ' WHERE ( ('.$whereCond;
        }else{
            $whereCond = array();
            if($active >= 0){
                $whereCond[] = 'sen.active = ' . intval($active);
            }
            if($eventGroup){
                $whereCond[] = 'sen.event_group = \''.$eventGroup.'\'';
            }
            if(count($whereCond) > 0) {
                $sql .= (' WHERE ' . implode(' AND ', $whereCond));
            }
        }
        return $sql;
    }

    protected function getSubSections(){
        global $wpdb;
        $query = 'SELECT active AS status, count(*) AS sensor_cr FROM '.WADA_Database::tbl_sensors().' GROUP BY active';
        $sensorCounts = $wpdb->get_results($query);

        if(count($sensorCounts) == 1){
            $sensorCounts[] = (object) (array('status' => ($sensorCounts[0]->status == 1 ? 0 : 1), 'sensor_cr' => 0));
        }

        $nrEnabled = $sensorCounts[1]->sensor_cr;
        $nrDisabled = $sensorCounts[0]->sensor_cr;
        if($sensorCounts[0]->status == 1){
            $nrEnabled = $sensorCounts[0]->sensor_cr;
            $nrDisabled = $sensorCounts[1]->sensor_cr;
        }

        $active = $this->getActiveFilterFromRequest();
        $basePage = 'wp-admin-audit-sensors';

        $all = new stdClass();
        $all->page = $basePage;
        $all->class = 'all';
        $all->param = 'active';
        $all->paramValue = -1;
        $all->default = true;
        $all->current = ($active == $all->paramValue);
        $all->title = __('All', 'wp-admin-audit');
        $all->count = ($nrEnabled + $nrDisabled);

        $enabled = new stdClass();
        $enabled->page = $basePage;
        $enabled->class = 'enabled';
        $enabled->param = 'active';
        $enabled->paramValue = 1;
        $enabled->default = false;
        $enabled->current = ($active == $enabled->paramValue);
        $enabled->title = __('Enabled', 'wp-admin-audit');
        $enabled->count = $nrEnabled;

        $disabled = new stdClass();
        $disabled->page = $basePage;
        $disabled->class = 'disabled';
        $disabled->param = 'active';
        $disabled->paramValue = 0;
        $disabled->default = false;
        $disabled->current = ($active == $disabled->paramValue);
        $disabled->title = __('Disabled', 'wp-admin-audit');
        $disabled->count = $nrDisabled;

        $sections = array();
        $sections[] = $all;
        $sections[] = $enabled;
        $sections[] = $disabled;

        return $sections;
    }

    protected function getEventGroupOptions(){
        $eventGroupOptions = array();
        $eventGroupOptions[''] = __('All event groups', 'wp-admin-audit');
        $eventGroupOptions[WADA_Sensor_Base::GRP_CORE] = __('Core', 'wp-admin-audit');
        $eventGroupOptions[WADA_Sensor_Base::GRP_MEDIA] = __('Media', 'wp-admin-audit');
        $eventGroupOptions[WADA_Sensor_Base::GRP_POST] = __('Post', 'wp-admin-audit');
        $eventGroupOptions[WADA_Sensor_Base::GRP_PLUGIN] = __('Plugin', 'wp-admin-audit');
        $eventGroupOptions[WADA_Sensor_Base::GRP_SETTING] = __('Setting', 'wp-admin-audit');
        $eventGroupOptions[WADA_Sensor_Base::GRP_THEME] = __('Theme', 'wp-admin-audit');
        $eventGroupOptions[WADA_Sensor_Base::GRP_USER] = __('User', 'wp-admin-audit');
        $eventGroupOptions[WADA_Sensor_Base::GRP_PLG_WADA] = __('WP Admin Audit', 'wp-admin-audit');

        global $wpdb;
        $sql = 'SELECT DISTINCT event_group '
                . 'FROM '.WADA_Database::tbl_sensors().' sen '
                . 'WHERE event_group <> \'PSEUDO\' AND event_group NOT IN (\''.implode("','", array_map('addslashes', array_keys($eventGroupOptions))).'\') '
                . 'ORDER BY event_group';
        $pluginEventGroups = $wpdb->get_results($sql);
        if($pluginEventGroups && count($pluginEventGroups)){
            $pluginEventGroups = array_column($pluginEventGroups, 'event_group');
            foreach($pluginEventGroups AS $plgEventGroup){
                switch($plgEventGroup){
                    case WADA_Sensor_Base::GRP_PLG_WC_PRODUCT:
                        $eventGroupOptions[WADA_Sensor_Base::GRP_PLG_WC_PRODUCT] = __('Woocommerce Product', 'wp-admin-audit');
                        break;
                    default:
                        WADA_Log::warning('getEventGroupOptions Unknown event group: '.$plgEventGroup);
                        $eventGroupOptions[sanitize_text_field($plgEventGroup)] = sanitize_text_field($plgEventGroup);
                }
            }
        }

        return $eventGroupOptions;
    }

    protected function getFilterControls(){
        $filterControls = array();

        $eventGroupFilter = new stdClass();
        $eventGroupFilter->type = 'select';
        $eventGroupFilter->value = $this->getEventGroupFilterFromRequest();
        $eventGroupFilter->field = 'event_group';
        $eventGroupFilter->label = null; // no need for label
        $eventGroupFilter->selectOptions = $this->getEventGroupOptions();

        $filterControls[] = $eventGroupFilter;
        //WADA_Log::debug('getFilterControls: '.print_r($filterControls, true));

        return $filterControls;
    }

    protected function displayAfterList(){
        echo '<div style="text-align: center;">';
        echo '<i>'.esc_html(__('Is an event critical to you missing?', 'wp-admin-audit')).'</i>';
        echo ' <a href="'.admin_url('/admin.php?page=wp-admin-audit-info&tab=tab-3&mode=sensor').'">'.esc_html(__('Let us know!', 'wp-admin-audit')).'</a>';
        echo '</div>';
        parent::displayAfterList();
    }

    public function no_items() {
        echo '<p>';
        if ( ! empty( $_REQUEST['s'] ) ) {
            $s = esc_html(wp_unslash(sanitize_text_field($_REQUEST['s'])));
            printf( __('No %s found for: %s', 'wp-admin-audit'), $this->_args['plural'], '<strong>' . $s . '</strong>' );
        } elseif ( $this->areFiltersActive() ) {
            printf( __('No %s found for filter selection', 'wp-admin-audit'), $this->_args['plural']);
        }
        echo '</p>';
        echo _e('Do you think we should add that sensor?').' <a href="'.admin_url('/admin.php?page=wp-admin-audit-info&tab=tab-3&mode=sensor').'">'.esc_html(__('Let us know!', 'wp-admin-audit')).'</a>';
    }

    function loadJavascriptActions(){ // override with extending functionality ?>
        <script type="text/javascript">
            function getAdditionalQueryVariables(){
                let data = {
                    event_group: jQuery('select[name=event_group]').val() || '',
                    active: list.__query(document.location.href, 'active') || ''
                };
                return data;
            }
            function getAdditionalParametersFromUrl(query){
                let data = {
                    event_group: list.__query( query, 'event_group' ) || '',
                    active: list.__query( query, 'active' ) || '',
                };
                return data;
            }
            function registerAdditionalListEvents(){
                jQuery('#event_group').on('change', function (e) {
                    let data = list.getQueryVariablesFromInputs();
                    list.update(data);
                });

                jQuery('.sensor-status-toggle').on('change', function (e) {
                    //console.log(e);
                    let id = parseInt(this.name.substr(6)); // taking name and stripping 'active' prefix
                    let active = jQuery(this).prop('checked');
                    if(id > 0){
                        toggleSensorStatus({'id':id, 'active':active});
                    }
                });
            }
            function unregisterAdditionalListEvents(){
                jQuery('#event_group').off('change');
                jQuery('.sensor-status-toggle').off('change');
            }
            function toggleSensorStatus(data){
                jQuery.ajax({
                    url: ajaxurl,
                    data: jQuery.extend({
                        <?php echo static::NONCE_NAME; ?>: jQuery('#<?php echo static::NONCE_NAME; ?>').val(),
                        action: '_wada_ajax_sensors_status_toggle'
                    }, data),
                    success: function (response) {
                        var response = jQuery.parseJSON(response);
                        if(response && response.success){
                            console.log(data);
                        }else{
                            jQuery('#active'+data.id).prop('checked', !data.active); // something went wrong saving, revert UI
                        }
                    }
                });
            }
            (function ($) {
                registerAdditionalListEvents();
            })(jQuery);
        </script>
        <?php
        parent::loadJavascriptActions(); // make sure base class also loads JS
    }

}
